# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
import unittest

from azure.cli.testsdk import ScenarioTest, record_only
import json
import os
from azure.cli.testsdk.scenario_tests import AllowLargeResponse


id_sql = '/subscriptions/38304e13-357e-405e-9e9a-220351dcce8c/resourceGroups/SQL-CLITEST-RG/providers/Microsoft.Compute/virtualMachines/sql-clitestvm-donotuse2'
item_id_sql = '/subscriptions/38304e13-357e-405e-9e9a-220351dcce8c/resourcegroups/sql-clitest-rg/providers/Microsoft.RecoveryServices/vaults/sql-clitestvault-donotuse/backupFabrics/Azure/protectionContainers/VMAppContainer;compute;sql-clitest-rg;sql-clitestvm-donotuse2/protectedItems/SQLDataBase;mssqlserver;msdb'
sub_sql = '38304e13-357e-405e-9e9a-220351dcce8c'
rg_sql = 'sql-clitest-rg'
vault_sql = 'sql-clitestvault-donotuse'
container_sql = 'VMAppContainer;Compute;sql-clitest-rg;sql-clitestvm-donotuse2'
container_friendly_sql = 'sql-clitestvm-donotuse2'
server_friendly_sql = 'sql-clitestvm-d'
item_auto_sql = 'SQLInstance;mssqlserver'
item1_sql = 'SQLDataBase;mssqlserver;msdb'
item1_sql_fname = 'msdb'
instance_name = 'sqlinstance;mssqlserver'


class BackupTests(ScenarioTest, unittest.TestCase):
    # SQL workload tests start here
    # Please make sure you have the following setup in place before running the tests -

    # For the tests using sql-clitestvm-donotuse2 and sql-clitestvault-donotuse -
    # Each test will register the container at the start and unregister at the end of the test
    # Make sure that the container is not already registered since the start of the test

    # Note: Archive test uses different subscription. Please comment them out when running the whole test suite at once. And run those tests individually.


    def register_container(self):
        # Check if the container is already registered to the vault, and if it is, if it's in soft-deleted state. Register/Re-register accordingly.
        existing_container = self.cmd('backup container show --backup-management-type AzureWorkload -g "{rg}" -v "{vault}" -n "{name}"').get_output_in_json()
        if existing_container:
            print("Found an existing container")
            if 'properties' in existing_container and \
                    'registrationStatus' in existing_container['properties'] and \
                    existing_container['properties']['registrationStatus'] == 'SoftDeleted':
                print("Container is soft-deleted - reregistering")
                self.cmd('backup container re-register --backup-management-type AzureWorkload --workload-type "{wt}" -g "{rg}" -v "{vault}" -c "{name}" --yes')
        else:
            print("Registering the container anew")
            self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 1)])

    def register_item(self):
        # Check if the item is already registered for backup. If not, register, if it is, undelete + reprotect as appropriate.
        existing_item = self.cmd('backup item show --backup-management-type AzureWorkload -g "{rg}" -v "{vault}" -c "{name}" -n "{item}"').get_output_in_json()
        if existing_item:
            print("Found an existing item")
            if 'properties' in existing_item and \
                    'isScheduledForDeferredDelete' in existing_item['properties'] and \
                    existing_item['properties']['isScheduledForDeferredDelete']:
                print("item was soft-deleted - undeleting")
                self.cmd('backup protection undelete --backup-management-type AzureWorkload --workload-type "{wt}" -c "{name}" -i "{item}" -g "{rg}" -v "{vault}"')
                time.sleep(10)
            
            existing_item = self.cmd('backup item show --backup-management-type AzureWorkload -g "{rg}" -v "{vault}" -c "{name}" -n "{item}"').get_output_in_json()
            if 'properties' in existing_item and \
                    'protectionState' in existing_item['properties'] and \
                    existing_item['properties']['protectionState'] == 'ProtectionStopped':
                print("item was in protection stopped state - resuming")
                self.cmd('backup protection resume --backup-management-type AzureWorkload --workload-type "{wt}" -g "{rg}" -v "{vault}" -c "{name}" -i "{item}" -p "{policy}"', checks=[
                    self.check("properties.entityFriendlyName", '{fitem}'),
                    self.check("properties.operation", "ConfigureBackup"),
                    self.check("properties.status", "Completed"),
                    self.check("resourceGroup", '{rg}')
                ])
        else:
            print("registering the item anew")
            self.cmd('backup protection enable-for-azurewl -v {vault} -g {rg} -p {policy} --protectable-item-type {pit} --protectable-item-name {item} --server-name {fname} --workload-type {wt}', checks=[
                self.check("properties.entityFriendlyName", '{fitem}'),
                self.check("properties.operation", "ConfigureBackup"),
                self.check("properties.status", "Completed"),
                self.check("resourceGroup", '{rg}')
            ])

    @record_only()
    def test_backup_wl_sql_container(self):

        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'fname': container_friendly_sql,
            'rg': rg_sql,
            'wt': 'MSSQL',
            'sub': sub_sql,
            'id': id_sql
        })

        self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id} ')

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 1)])

        container_json = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check('properties.friendlyName', '{fname}'),
            self.check('properties.healthStatus', 'Healthy'),
            self.check('properties.registrationStatus', 'Registered'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.kwargs['container_name'] = container_json['name']

        self.cmd('backup container show -n {container_name} -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check('properties.friendlyName', '{fname}'),
            self.check('properties.healthStatus', 'Healthy'),
            self.check('properties.registrationStatus', 'Registered'),
            self.check('name', '{container_name}'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.assertIn(self.kwargs['vault'].lower(), container_json['id'].lower())
        self.assertIn(self.kwargs['name'].lower(), container_json['name'].lower())

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?properties.friendlyName == '{fname}'])", 1)])

        self.cmd('backup container re-register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} -y --container-name {name}')

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 1)])

        self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @record_only()
    def test_backup_wl_sql_policy(self):

        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'fname': container_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'default': 'HourlyLogBackup',
            'rg': rg_sql,
            'item': item1_sql,
            'id': id_sql,
            'item_id': item_id_sql,
            'pit': 'SQLDataBase',
            'policy_new': self.create_random_name('clitest-policy', 24)
        })
        self.kwargs['policy1_json'] = self.cmd('backup policy show -g {rg} -v {vault} -n {policy}', checks=[
            self.check('name', '{policy}'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.kwargs['policy_json'] = json.dumps(self.kwargs['policy1_json'], separators=(',', ':')).replace('\'', '\\\'').replace('"', '\\"')

        self.cmd("backup policy create -g {rg} -v {vault} --policy {policy_json} --backup-management-type AzureWorkload --workload-type {wt} --name {policy_new}", checks=[
            self.check('name', '{policy_new}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup policy list -g {rg} -v {vault}', checks=[
            self.check("length([?name == '{default}'])", 1),
            self.check("length([?name == '{policy}'])", 1),
            self.check("length([?name == '{policy_new}'])", 1)
        ])

        self.kwargs['policy1_json']['properties']['settings']['isCompression'] = 'true'
        self.kwargs['policy1_json']['properties']['settings']['issqlcompression'] = 'true'
        self.kwargs['policy1_json'] = json.dumps(self.kwargs['policy1_json'], separators=(',', ':')).replace('\'', '\\\'').replace('"', '\\"')

        self.cmd("backup policy set -g {rg} -v {vault} --policy {policy1_json} -n {policy_new}", checks=[
            self.check('name', '{policy_new}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd("backup policy set -g {rg} -v {vault} --backup-management-type AzureWorkload --fix-for-inconsistent-items -n {policy_new}", checks=[
            self.check('name', '{policy_new}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup policy show -g {rg} -v {vault} -n {policy_new}', checks=[
            self.check('name', '{policy_new}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup policy delete -g {rg} -v {vault} -n {policy_new}')

        self.cmd('backup policy list -g {rg} -v {vault}', checks=[
            self.check("length([?name == '{default}'])", 1),
            self.check("length([?name == '{policy}'])", 1),
            self.check("length([?name == '{policy_new}'])", 0)
        ])

    @record_only()
    def test_backup_wl_sql_protectable_item(self):

        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'default': 'HourlyLogBackup',
            'rg': rg_sql,
            'item': item1_sql,
            'id': id_sql,
            'item_id': item_id_sql,
            'pit': 'SQLDataBase',
            'protectable_item_name': item1_sql_fname,
            'pit_hana': 'SAPHanaDatabase'
        })

        # self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')
        self.register_container()

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 1)])

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --query properties.friendlyName --backup-management-type AzureWorkload').get_output_in_json()

        self.cmd('backup protectable-item list -g {rg} --vault-name {vault} --workload-type {wt}', checks=[
            self.check("length([?properties.friendlyName == 'master'])", 1)
        ])

        self.cmd('backup protectable-item show -g {rg} --vault-name {vault} --name "master" --workload-type {wt} --protectable-item-type {pit} --server-name {fname}', checks=[
            self.check('properties.friendlyName', 'master'),
            self.check('properties.protectableItemType', '{pit}'),
            self.check('properties.serverName', '{fname}'),
            self.check('resourceGroup', '{rg}')
        ])

        # self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @record_only()
    def test_backup_wl_sql_rp(self):
        resource_group = rg_sql.lower()
        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'rg': resource_group,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'item': item1_sql,
            'pit': 'SQLDatabase',
            'item_id': item_id_sql,
            'id': id_sql,
            'fitem': item1_sql_fname
        })

        self.register_container()
        self.register_item()

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {name} -i {item} --workload-type {wt} --query [].name', checks=[
            self.check("length(@)", 1)
        ])

        rp1_json = self.cmd('backup recoverypoint show-log-chain -g {rg} -v {vault} -c {name} -i {item} --workload-type {wt}').get_output_in_json()
        self.assertIn(vault_sql.lower(), rp1_json[0]['id'].lower())
        self.assertIn(container_sql.lower(), rp1_json[0]['id'].lower())

        rp2_json = self.cmd('backup recoverypoint show-log-chain -g {rg} -v {vault} -c {name} -i {item} --workload-type {wt}').get_output_in_json()
        self.assertIn(vault_sql.lower(), rp2_json[0]['id'].lower())
        self.assertIn(container_sql.lower(), rp2_json[0]['id'].lower())

        self.cmd('backup protection disable -v {vault} -g {rg} -c {container1} --backup-management-type AzureWorkload --workload-type {wt} -i {item} -y --delete-backup-data true')

        # self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @record_only()
    @unittest.skip("Covered by other scenarios")
    def test_backup_wl_sql_auto_protection(self):
        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'default': 'HourlyLogBackup',
            'rg': rg_sql,
            'item': item_auto_sql,
            'fitem': item_auto_sql.split(';')[-1],
            'id': id_sql,
            'item_id': item_id_sql,
            'pit': 'SQLInstance'
        })

        self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 1)])

        self.cmd('backup protection auto-enable-for-azurewl -v {vault} -g {rg} -p {policy} --protectable-item-name {item} --protectable-item-type {pit} --server-name {fname} --workload-type {wt}')

        protectable_item_json = self.cmd('backup protectable-item show -v {vault} -g {rg} -n {item} --protectable-item-type {pit} --server-name {fname} --workload-type {wt}', checks=[
            self.check("properties.isAutoProtected", True)]).get_output_in_json()

        self.assertIn(self.kwargs['policy'], protectable_item_json['properties']['autoProtectionPolicy'])

        self.cmd('backup protection auto-disable-for-azurewl -v {vault} -g {rg} --protectable-item-name {item} --protectable-item-type {pit} --server-name {fname} --workload-type {wt}')

        self.cmd('backup protectable-item show -v {vault} -g {rg} -n {item} --protectable-item-type {pit} --server-name {fname} --workload-type {wt}', checks=[
            self.check("properties.isAutoProtected", False),
            self.check("properties.autoProtectionPolicy", None)])

        self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 0)])

    @record_only()
    def test_backup_wl_sql_item(self):
        resource_group = rg_sql.lower()
        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'rg': resource_group,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'item': item1_sql,
            'pit': 'SQLDatabase',
            'item_id': item_id_sql,
            'id': id_sql,
            'fitem': item1_sql_fname
        })

        # self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 1)])

        # self.cmd('backup protection enable-for-azurewl -v {vault} -g {rg} -p {policy} --protectable-item-type {pit} --protectable-item-name {item} --server-name {fname} --workload-type {wt}')
        self.register_container()
        self.register_item()

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        item1_json = self.cmd('backup item show -g {rg} -v {vault} -c {name} -n {item} --backup-management-type AzureWorkload --workload-type {wt}', checks=[
            self.check('properties.friendlyName', '{fitem}'),
            self.check('properties.protectedItemHealthStatus', 'IRPending'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.assertIn(self.kwargs['vault'].lower(), item1_json['id'].lower())
        self.assertIn(self.kwargs['fname'].lower(), item1_json['properties']['containerName'].lower())
        self.assertIn(self.kwargs['fname'].lower(), item1_json['properties']['sourceResourceId'].lower())
        self.assertIn(self.kwargs['policy'].lower(), item1_json['properties']['policyId'].lower())

        self.kwargs['container1_fullname'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        self.cmd('backup item show -g {rg} -v {vault} -c {container1_fullname} -n {item} --backup-management-type AzureWorkload --workload-type {wt}', checks=[
            self.check('properties.friendlyName', '{fitem}'),
            self.check('properties.protectedItemHealthStatus', 'IRPending'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.kwargs['item1_fullname'] = item1_json['name']

        self.cmd('backup item show -g {rg} -v {vault} -c {container1_fullname} -n {item1_fullname} --backup-management-type AzureWorkload --workload-type SAPHanaDatabase', checks=[
            self.check('properties.friendlyName', '{fitem}'),
            self.check('properties.protectedItemHealthStatus', 'IRPending'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup item list -g {rg} -v {vault} -c {container1} --backup-management-type AzureWorkload --workload-type SQLDataBase', checks=[
            self.check("length([?properties.friendlyName == '{fitem}'])", 1)
        ])

        self.cmd('backup item list -g {rg} -v {vault} -c {container1_fullname} --backup-management-type AzureWorkload --workload-type SQLDataBase', checks=[
            self.check("length([?properties.friendlyName == '{fitem}'])", 1)
        ])

        self.cmd('backup item list -g {rg} -v {vault} --backup-management-type AzureWorkload --workload-type SQLDataBase', checks=[
            self.check("length([?properties.friendlyName == '{fitem}'])", 1)
        ])

        self.cmd('backup item set-policy -g {rg} -v {vault} -c {container1} -n {item1_fullname} -p {policy} --backup-management-type AzureWorkload --workload-type SQLDataBase', checks=[
            self.check("properties.entityFriendlyName", '{fitem}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        item1_json = self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item} --backup-management-type AzureWorkload --workload-type SQLDataBase').get_output_in_json()
        self.assertIn("HourlyLogBackup".lower(), item1_json['properties']['policyId'].lower())

        self.cmd('backup protection disable -v {vault} -g {rg} -c {container1} --backup-management-type AzureWorkload --workload-type {wt} -i {item} -y --delete-backup-data true')

        # self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @record_only()
    def test_backup_wl_sql_protection(self):
        resource_group = rg_sql.lower()
        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'rg': resource_group,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'item': item1_sql,
            'pit': 'SQLDatabase',
            'item_id': item_id_sql,
            'id': id_sql,
            'fitem': item1_sql_fname 
        })

        # self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 1)])

        # self.cmd('backup protection enable-for-azurewl -v {vault} -g {rg} -p {policy} --protectable-item-type {pit} --protectable-item-name {item} --server-name {fname} --workload-type {wt}', checks=[
        #     self.check("properties.entityFriendlyName", '{fitem}'),
        #     self.check("properties.operation", "ConfigureBackup"),
        #     self.check("properties.status", "Completed"),
        #     self.check("resourceGroup", '{rg}')
        # ])
        self.register_container()
        self.register_item()

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        self.kwargs['backup_job'] = self.cmd('backup protection backup-now -v {vault} -g {rg} -i {item} -c {name} --backup-type Full --enable-compression false', checks=[
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.assertIn("Backup", self.kwargs['backup_job']['properties']['operation'])

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job}')

        self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item} --backup-management-type AzureWorkload', checks=[
            self.check('properties.friendlyName', '{fitem}'),
            self.check('properties.protectedItemHealthStatus', 'Healthy'),
            self.check('properties.protectionState', 'Protected'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup protection disable -v {vault} -g {rg} -i {item} -c {name} --backup-management-type AzureWorkload -y', checks=[
            self.check("properties.entityFriendlyName", '{fitem}'),
            self.check("properties.operation", "DisableBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item} --backup-management-type AzureWorkload', checks=[
            self.check("properties.friendlyName", '{fitem}'),
            self.check("properties.protectionState", "ProtectionStopped"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup protection disable -v {vault} -g {rg} -c {container1} --backup-management-type AzureWorkload --workload-type {wt} -i {item} -y --delete-backup-data true')

        # self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @record_only()
    def test_backup_wl_sql_restore(self):
        resource_group = rg_sql.lower()
        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'rg': resource_group,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'item': item1_sql,
            'fitem': item1_sql_fname,
            'id': id_sql,
            'pit': 'SQLDatabase',
            'item_id': item_id_sql,
            'titem': item1_sql_fname + '_restored',
            'tinstance_name': instance_name
        })

        # self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 1)])

        # self.cmd('backup protection enable-for-azurewl -v {vault} -g {rg} -p {policy} --protectable-item-type {pit} --protectable-item-name {item} --server-name {fname} --workload-type {wt}', checks=[
        #     self.check("properties.entityFriendlyName", '{fitem}'),
        #     self.check("properties.operation", "ConfigureBackup"),
        #     self.check("properties.status", "Completed"),
        #     self.check("resourceGroup", '{rg}')
        # ])
        self.register_container()
        self.register_item()

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        self.kwargs['backup_job'] = self.cmd('backup protection backup-now -v {vault} -g {rg} -i {item} -c {name} --backup-type Full --enable-compression false', checks=[
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.assertIn("Backup", self.kwargs['backup_job']['properties']['operation'])

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job}')

        self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item} --backup-management-type AzureWorkload', checks=[
            self.check('properties.friendlyName', '{fitem}'),
            self.check('properties.protectedItemHealthStatus', 'Healthy'),
            self.check('properties.protectionState', 'Protected'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.kwargs['rp'] = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {name} -i {item} --workload-type {wt} --query [0]').get_output_in_json()

        self.kwargs['rp'] = self.kwargs['rp']['name']

        self.kwargs['rc'] = json.dumps(self.cmd('backup recoveryconfig show --vault-name {vault} -g {rg} --restore-mode AlternateWorkloadRestore --rp-name {rp} --item-name {item} --container-name {container1} --target-item-name {titem} --target-server-type SQLInstance --target-server-name {fname} --workload-type {wt} --target-instance-name {tinstance_name}').get_output_in_json(), separators=(',', ':'))
        with open("recoveryconfig_sql_restore.json", "w") as f:
            f.write(self.kwargs['rc'])

        self.kwargs['backup_job'] = self.cmd('backup restore restore-azurewl --vault-name {vault} -g {rg} --recovery-config recoveryconfig_sql_restore.json', checks=[
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job}')

        self.kwargs['rc'] = json.dumps(self.cmd('backup recoveryconfig show --vault-name {vault} -g {rg} --restore-mode OriginalWorkloadRestore --item-name {item} --container-name {container1} --rp-name {rp}').get_output_in_json(), separators=(',', ':'))
        with open("recoveryconfig_sql_restore.json", "w") as f:
            f.write(self.kwargs['rc'])

        self.kwargs['backup_job'] = self.cmd('backup restore restore-azurewl --vault-name {vault} -g {rg} --recovery-config recoveryconfig_sql_restore.json', checks=[
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job}')

        self.cmd('backup protection disable -v {vault} -g {rg} -c {name} --backup-management-type AzureWorkload --workload-type {wt} -i {item} -y --delete-backup-data true')

        # self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @record_only()
    def test_backup_wl_sql_restore_as_files(self):
        resource_group = rg_sql.lower()
        self.kwargs.update({
            'vault': vault_sql,
            'name': container_sql,
            'rg': resource_group,
            'fname': server_friendly_sql,
            'policy': 'HourlyLogBackup',
            'wt': 'MSSQL',
            'sub': sub_sql,
            'item': item1_sql,
            'fitem': item1_sql_fname,
            'id': id_sql,
            'pit': 'SQLDatabase',
            'item_id': item_id_sql,
            'titem': item1_sql_fname + '_restored'
        })
        # self.cmd('backup container register -v {vault} -g {rg} --backup-management-type AzureWorkload --workload-type {wt} --resource-id {id}')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 1)])

        # self.cmd('backup protection enable-for-azurewl -v {vault} -g {rg} -p {policy} --protectable-item-type {pit} --protectable-item-name {item} --server-name {fname} --workload-type {wt}', checks=[
        #     self.check("properties.entityFriendlyName", '{fitem}'),
        #     self.check("properties.operation", "ConfigureBackup"),
        #     self.check("properties.status", "Completed"),
        #     self.check("resourceGroup", '{rg}')
        # ])
        self.register_container()
        self.register_item()

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        self.kwargs['backup_job'] = self.cmd('backup protection backup-now -v {vault} -g {rg} -i {item} -c {name} --backup-type Full --enable-compression false', checks=[
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.assertIn("Backup", self.kwargs['backup_job']['properties']['operation'])

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job}')

        self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item} --backup-management-type AzureWorkload', checks=[
            self.check('properties.protectedItemHealthStatus', 'Healthy'),
            self.check('properties.protectionState', 'Protected'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.kwargs['rp'] = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {name} -i {item} --workload-type {wt} --query [0]').get_output_in_json()
        self.kwargs['rp'] = self.kwargs['rp']['name']

        self.kwargs['rc'] = json.dumps(self.cmd('backup recoveryconfig show --vault-name {vault} -g {rg} --restore-mode RestoreAsFiles --rp-name {rp} --filepath "C:\" --target-container-name {container1} --item-name {item} --container-name {container1}  --workload-type {wt}').get_output_in_json(), separators=(',', ':'))
        with open("recoveryconfig_sql_raf.json", "w") as f:
            f.write(self.kwargs['rc'])

        self.kwargs['backup_job'] = self.cmd('backup restore restore-azurewl --vault-name {vault} -g {rg} --recovery-config recoveryconfig_sql_raf.json', checks=[
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job}')

        self.cmd('backup protection disable -v {vault} -g {rg} -c {name} --backup-management-type AzureWorkload --workload-type {wt} -i {item} -y --delete-backup-data true')

        # self.cmd('backup container unregister -v {vault} -g {rg} -c {name} -y')

        # self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
        #     self.check("length([?name == '{name}'])", 0)])

    @AllowLargeResponse()
    @record_only()
    def test_backup_wl_sql_crr(self):
        self.kwargs.update({
            'vault': "sql-clitest-vault",
            'name': "VMAppContainer;Compute;sql-clitest-rg;sql-clitest-vm",
            'fname': "sql-clitest-vm",
            'wt': 'MSSQL',
            'sub': "vsarg-MABPortalTestAutomation_NOB",
            'rg': "sql-clitest-rg",
            'item': "SQLDataBase;mssqlserver;msdb",
            'fitem': "msdb",
            'tvault': "clitest-vault-secondary-donotuse",
            'trg': "clitest-rg-donotuse",
            'tcontainer': "clitest-sql-secondary-donotuse",
            'tserver': "clitest-sql-sec",
            'tpit': 'SQLInstance',
            'titem': 'msdb_restored'
        })
        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type AzureWorkload', checks=[
            self.check("length([?name == '{name}'])", 1)])

        self.kwargs['container1'] = self.cmd('backup container show -n {name} -v {vault} -g {rg} --backup-management-type AzureWorkload --query name').get_output_in_json()

        self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item} --backup-management-type AzureWorkload', checks=[
            self.check('properties.friendlyName', '{fitem}'),
            self.check('properties.protectionState', 'Protected'),
            self.check('resourceGroup', '{rg}')
        ])

        self.kwargs['rp'] = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {name} -i {item} --workload-type {wt} --use-secondary-region --query [0]').get_output_in_json()

        self.kwargs['rp'] = self.kwargs['rp']['name']

        #SQL CRR ALR Restore
        self.kwargs['rc'] = json.dumps(self.cmd('backup recoveryconfig show --vault-name {vault} -g {rg} --restore-mode AlternateWorkloadRestore --rp-name {rp} --item-name {item} --container-name {container1} --target-item-name {titem} --target-server-type SQLInstance --target-server-name {tserver} --target-container-name {tcontainer} --workload-type {wt} --target-vault-name {tvault} --target-resource-group {trg}').get_output_in_json(), separators=(',', ':'))
        with open("recoveryconfig_sql_crr.json", "w") as f:
            f.write(self.kwargs['rc'])

        self.kwargs['backup_job'] = self.cmd('backup restore restore-azurewl --vault-name {vault} -g {rg} --recovery-config recoveryconfig_sql_crr.json --use-secondary-region', checks=[
            self.check("properties.operation", "CrossRegionRestore"),
            self.check("properties.status", "InProgress")
        ]).get_output_in_json()

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        self.cmd('backup job wait -v {vault} -g {rg} -n {job} --use-secondary-region')

        #SQL CRR RAF Restore
        self.kwargs['rc'] = json.dumps(self.cmd('backup recoveryconfig show --vault-name {vault} -g {rg} --restore-mode restoreasfiles --rp-name {rp} --item-name {item} --container-name {container1} --target-container-name {tcontainer} --workload-type {wt} --target-vault-name {tvault} --target-resource-group {trg} --filepath "C:\"').get_output_in_json(), separators=(',', ':'))
        with open("recoveryconfig_sql_crr.json", "w") as f:
            f.write(self.kwargs['rc'])

        self.kwargs['backup_job'] = self.cmd('backup restore restore-azurewl --vault-name {vault} -g {rg} --recovery-config recoveryconfig_sql_crr.json --use-secondary-region', checks=[
            self.check("properties.operation", "CrossRegionRestore"),
            self.check("properties.status", "InProgress")
        ]).get_output_in_json()

        self.kwargs['job'] = self.kwargs['backup_job']['name']

        # self.cmd('backup job wait -v {vault} -g {rg} -n {job} --use-secondary-region')

    @AllowLargeResponse()
    @record_only()
    def test_backup_wl_sql_archive (self):
        self.kwargs.update({
            'vault': "archiveccyvault1",
            'rg': "ArchiveResourceGroup",
            'sub': "AzureBackup_Functional_Testing",
            'item': "SQLDataBase;mssqlserver;msdb",
            'container': "VMAppContainer;compute;archiveresourcegroup;archsqlccyvm2"
        })
        
        # Getting the recovery point IDs (names) and storing it in a list
        rp_names = self.cmd('backup recoverypoint list --backup-management-type AzureWorkload --workload-type MSSQL -g {rg} -v {vault} -c {container} -i {item}', checks=[
        ]).get_output_in_json()

        self.kwargs['rp1'] = rp_names[0]['name']
        self.kwargs['rp1_tier'] = rp_names[0]['tierType']
        self.kwargs['rp1_is_ready_for_move'] = rp_names[0]['properties']['recoveryPointMoveReadinessInfo']['ArchivedRP']['isReadyForMove']
        
        # Check Archivable Recovery Points 
        self.cmd('backup recoverypoint list -g {rg} -v {vault} -i {item} -c {container} --backup-management-type AzureWorkload --is-ready-for-move {rp1_is_ready_for_move} --target-tier VaultArchive --query [0]', checks=[
            self.check("resourceGroup", '{rg}'),
            self.check("properties.recoveryPointMoveReadinessInfo.ArchivedRP.isReadyForMove", '{rp1_is_ready_for_move}')
        ])

        # Get Archived Recovery Points 
        self.cmd('backup recoverypoint list -g {rg} -v {vault} -i {item} -c {container} --backup-management-type AzureWorkload --tier {rp1_tier} --query [0]', checks=[
            self.check("tierType", '{rp1_tier}'),
            self.check("resourceGroup", '{rg}')
        ])

        is_move = False
        for i in rp_names:
            if i['tierType']=="VaultStandard" and i['properties']['recoveryPointMoveReadinessInfo']['ArchivedRP']['isReadyForMove']==True:
                self.kwargs['rp_move'] = i['name']
                is_move = True
                break
        
        if is_move:
            # # Move Recovery points
            self.cmd('backup recoverypoint move -g {rg} -v {vault} -i {item} -c {container} --source-tier VaultStandard --destination-tier VaultArchive --name {rp_move}', checks=[
                self.check("properties.entityFriendlyName", 'msdb [archsqlccyvm2]'),
                self.check("resourceGroup", '{rg}'),
                self.check("properties.operation", "MoveRecoveryPoint"),
                self.check("properties.status", "Completed")
            ])
        
        # Getting the recovery point ID in VaultArchive tier
        self.kwargs['rp_restore'] = self.cmd('backup recoverypoint list --backup-management-type AzureWorkload --workload-type MSSQL -g {rg} -v {vault} -c {container} -i {item} --tier VaultArchive --query [0]').get_output_in_json()
        self.kwargs['rp_restore'] = self.kwargs['rp_restore']['name']

        # # Integrated Restore
        self.kwargs['rc'] = json.dumps(self.cmd('backup recoveryconfig show --vault-name {vault} -g {rg} --restore-mode OriginalWorkloadRestore --item-name {item} --container-name {container} --rp-name {rp_restore}').get_output_in_json(), separators=(',', ':'))
        with open("recoveryconfig_sql_archive.json", "w") as f:
            f.write(self.kwargs['rc'])

        # # Trigger Restore
        self.cmd('backup restore restore-azurewl -g {rg} -v {vault} --recovery-config recoveryconfig_sql_archive.json --rehydration-priority High', checks=[
            self.check("properties.operation", "RestoreWithRehydrate"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()


    @AllowLargeResponse()
    @record_only()
    def test_backup_wl_reconfigure(self):
        self.kwargs.update({
            'resource_group': 'zubairRG',
            'vault1': 'zimmut-ccy-6',
            'vault2': 'zimmut-ccy-5',
            'container': 'sql-migration-vm2',
            'item': 'master',
            'policy_name': 'HourlyLogBackup'
        })

        self.cmd('backup vault show -g "{resource_group}" -n "{vault1}"', checks=[
            self.check('name', '{vault1}'),
            self.check('resourceGroup', '{resource_group}')
        ])

        self.cmd('backup vault show -g "{resource_group}" -n "{vault2}"', checks=[
            self.check('name', '{vault2}'),
            self.check('resourceGroup', '{resource_group}')
        ])

        # Verify container and items exist in vault1
        self.cmd('backup container show -g "{resource_group}" -v "{vault1}" -n "{container}" --backup-management-type AzureWorkload', checks=[
            self.check('properties.friendlyName', '{container}'),
            self.check('properties.registrationStatus', 'Registered')
        ])

        self.cmd('backup item show -g "{resource_group}" -v "{vault1}" -c "{container}" -n "{item}" --backup-management-type AzureWorkload --workload-type MSSQL', checks=[
            self.check('properties.friendlyName', '{item}'),
            self.check('properties.protectionState', 'Protected')
        ])

        # Verify policy exists in vault2
        self.cmd('backup policy show -g "{resource_group}" -v "{vault2}" -n "{policy_name}"', checks=[
            self.check('name', '{policy_name}'),
            self.check('resourceGroup', '{resource_group}')
        ])

        self.cmd('backup protection reconfigure -g "{resource_group}" -v "{vault1}" -c "{container}" -i "{item}" --backup-management-type "AzureWorkload" --workload-type "MSSQL" --new-rg "{resource_group}" --new-vault-name "{vault2}" --new-policy-name "{policy_name}"', checks=[
            self.check('properties.operation', 'ConfigureBackup'),
            self.check('properties.status', 'Completed')
        ])

        # Clean up - delete the backup item from vault2
        # self.cmd('backup protection disable -g "{resource_group}" -v "{vault2}" -c "{container}" -i "{item}" --backup-management-type AzureWorkload --workload-type MSSQL --delete-backup-data true --yes')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

from azure.cli.testsdk import (ResourceGroupPreparer, ScenarioTest)
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.core.exceptions import ResourceNotFoundError
from azure.cli.command_modules.appconfig.tests.latest._test_utils import CredentialResponseSanitizer, get_resource_name_prefix

class AppconfigReplicaLiveScenarioTest(ScenarioTest):

    def __init__(self, *args, **kwargs):
        kwargs["recording_processors"] = kwargs.get("recording_processors", []) + [CredentialResponseSanitizer()]
        super().__init__(*args, **kwargs)

    @ResourceGroupPreparer(parameter_name_for_location='location')
    @AllowLargeResponse()
    def test_azconfig_replica_mgmt(self, resource_group, location):
        store_name_prefix = get_resource_name_prefix('ReplicaStore')
        replica_name_prefix = get_resource_name_prefix('Replica')
        config_store_name = self.create_random_name(prefix=store_name_prefix, length=24)
        replica_name = self.create_random_name(prefix=replica_name_prefix, length=24)

        store_location = 'eastus'
        replica_location = 'westus'
        sku = 'standard'
        tag_key = "key"
        tag_value = "value"
        tag = tag_key + '=' + tag_value
        structured_tag = {tag_key: tag_value}
        system_assigned_identity = '[system]'

        self.kwargs.update({
            'config_store_name': config_store_name,
            'replica_name': replica_name,
            'rg_loc': store_location,
            'replica_loc': replica_location,
            'rg': resource_group,
            'sku': sku,
            'tags': tag,
            'identity': system_assigned_identity,
            'retention_days': 1,
            'enable_purge_protection': False
        })

        store = self.cmd(
            'appconfig create -n {config_store_name} -g {rg} -l {rg_loc} --sku {sku} --tags {tags} --assign-identity {identity} --retention-days {retention_days} --enable-purge-protection {enable_purge_protection}',
            checks=[self.check('name', '{config_store_name}'),
                    self.check('location', '{rg_loc}'),
                    self.check('resourceGroup', resource_group),
                    self.check('provisioningState', 'Succeeded'),
                    self.check('sku.name', sku),
                    self.check('tags', structured_tag),
                    self.check('identity.type', 'SystemAssigned'),
                    self.check('softDeleteRetentionInDays', '{retention_days}'),
                    self.check('enablePurgeProtection', '{enable_purge_protection}')]).get_output_in_json()

        self.cmd('appconfig replica create -s {config_store_name} -g {rg} -l {replica_loc} -n {replica_name}',
                 checks=[self.check('name', '{replica_name}'),
                         self.check('location', '{replica_loc}'),
                         self.check('resourceGroup', resource_group),
                         self.check('provisioningState', 'Succeeded')])
        
        self.cmd('appconfig replica show -s {config_store_name} -g {rg} -n {replica_name}',
                 checks=[self.check('name', '{replica_name}'),
                         self.check('location', '{replica_loc}'),
                         self.check('resourceGroup', resource_group),
                         self.check('provisioningState', 'Succeeded')])

        self.cmd('appconfig replica list -s {config_store_name}',
                 checks=[self.check('[0].name', '{replica_name}'),
                         self.check('[0].location', '{replica_loc}'),
                         self.check('[0].resourceGroup', resource_group),
                         self.check('[0].provisioningState', 'Succeeded')])

        self.cmd('appconfig replica delete -s {config_store_name} -g {rg} -n {replica_name} -y')

        with self.assertRaisesRegex(ResourceNotFoundError, f"The replica '{replica_name}' for App Configuration '{config_store_name}' not found."):
            self.cmd('appconfig replica show -s {config_store_name} -g {rg} -n {replica_name}')

        self.cmd('appconfig delete -n {config_store_name} -g {rg} -y')



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

import os
import sphinx

BUILD_DIR = '_build'
FORMAT = 'xml'

argv = ['sphinx-build', '', 'xml', os.getcwd(), BUILD_DIR]
sphinx.make_main(argv)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access, line-too-long

from azure.cli.core.aaz import has_value, AAZStrArg
from azure.cli.command_modules.monitor.actions import AAZCustomListArg
from azure.cli.command_modules.monitor.aaz.latest.monitor.activity_log.alert._update \
    import Update as _ActivityLogAlertUpdate
from azure.cli.command_modules.monitor.operations.activity_log_alerts import (
    process_condition_parameter_for_alert,
)


class ActivityLogAlertUpdate(_ActivityLogAlertUpdate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.action_groups._registered = False
        args_schema.scopes._registered = False
        args_schema.condition = AAZCustomListArg(
            options=["--condition", "-c"],
            help="The condition that will cause the alert rule to activate. "
                 "The format is FIELD=VALUE[ and FIELD=VALUE...]" + '''
        The possible values for the field are 'resourceId', 'category', 'caller',
        'level', 'operationName', 'resourceGroup', 'resourceProvider', 'status',
        'subStatus', 'resourceType', or anything beginning with 'properties'.
        '''
        )
        args_schema.condition.Element = AAZStrArg()
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        process_condition_parameter_for_alert(args)
        if not has_value(args.all_of):
            args.all_of.append({
                "field": "category",
                "equals": "ServiceHealth",
            })
        else:
            current_all_of = args.all_of.to_serialized_data()
            category_found = False
            for item in current_all_of:
                if item.get("field", None) == "category":
                    category_found = True
                    break
            if not category_found:
                args.all_of.append({
                    "field": "category",
                    "equals": "ServiceHealth",
                })



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import os
from azure.cli.core.azclierror import BadRequestError

_config = None


def get_config_json():
    global _config  # pylint:disable=global-statement
    if _config is not None:
        return _config
    script_dir = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(script_dir, "config.json"), "r") as f:
        try:
            _config = json.load(f)
            return _config
        except ValueError:
            raise BadRequestError("Invalid json file. Make sure that the json file content is properly formatted.")


def get_cloud(cmd):
    config = get_config_json()
    return config[cmd.cli_ctx.cloud.name]


def get_cloud_cluster(cmd, location, subscription_id):
    try:
        cloud = get_cloud(cmd)
        clusters = cloud[location]
    except KeyError:
        clusters = None
    if clusters is not None:
        for cluster in clusters:
            if cloud[cluster] is not None:
                if subscription_id in cloud[cluster]["subscriptions"]:
                    return cloud[cluster]
    return



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Recycle(AAZCommand):
    """Recycle a specific instance in a managed instance App Service plan.

    :example: Recycle a specific worker instance in a managed instance App Service plan.
        az appservice plan managed-instance instance recycle --name MyAppServicePlan --resource-group MyResourceGroup --worker-name MyWorkerInstance
    """

    _aaz_info = {
        "version": "2025-03-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.web/serverfarms/{}/workers/{}/recycleinstance", "2025-03-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["--name"],
            help="Name of the App Service plan.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.worker_name = AAZStrArg(
            options=["--worker-name"],
            help="Name of worker machine.",
            required=True,
            id_part="child_name_1",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9]+$",
            ),
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.AppServicePlansRecycleManagedInstanceWorker(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AppServicePlansRecycleManagedInstanceWorker(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/serverfarms/{name}/workers/{workerName}/recycleinstance",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "workerName", self.ctx.args.worker_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.created_time = AAZStrType(
                serialized_name="createdTime",
            )
            _schema_on_200.errors = AAZListType()
            _schema_on_200.expiration_time = AAZStrType(
                serialized_name="expirationTime",
            )
            _schema_on_200.geo_master_operation_id = AAZStrType(
                serialized_name="geoMasterOperationId",
            )
            _schema_on_200.id = AAZStrType()
            _schema_on_200.modified_time = AAZStrType(
                serialized_name="modifiedTime",
            )
            _schema_on_200.name = AAZStrType()
            _schema_on_200.status = AAZStrType()

            errors = cls._schema_on_200.errors
            errors.Element = AAZObjectType()
            _RecycleHelper._build_schema_error_entity_read(errors.Element)

            return cls._schema_on_200


class _RecycleHelper:
    """Helper class for Recycle"""

    _schema_error_entity_read = None

    @classmethod
    def _build_schema_error_entity_read(cls, _schema):
        if cls._schema_error_entity_read is not None:
            _schema.code = cls._schema_error_entity_read.code
            _schema.details = cls._schema_error_entity_read.details
            _schema.extended_code = cls._schema_error_entity_read.extended_code
            _schema.inner_errors = cls._schema_error_entity_read.inner_errors
            _schema.message = cls._schema_error_entity_read.message
            _schema.message_template = cls._schema_error_entity_read.message_template
            _schema.parameters = cls._schema_error_entity_read.parameters
            _schema.target = cls._schema_error_entity_read.target
            return

        cls._schema_error_entity_read = _schema_error_entity_read = AAZObjectType()

        error_entity_read = _schema_error_entity_read
        error_entity_read.code = AAZStrType()
        error_entity_read.details = AAZListType()
        error_entity_read.extended_code = AAZStrType(
            serialized_name="extendedCode",
        )
        error_entity_read.inner_errors = AAZListType(
            serialized_name="innerErrors",
        )
        error_entity_read.message = AAZStrType()
        error_entity_read.message_template = AAZStrType(
            serialized_name="messageTemplate",
        )
        error_entity_read.parameters = AAZListType()
        error_entity_read.target = AAZStrType()

        details = _schema_error_entity_read.details
        details.Element = AAZObjectType()
        cls._build_schema_error_entity_read(details.Element)

        inner_errors = _schema_error_entity_read.inner_errors
        inner_errors.Element = AAZObjectType()
        cls._build_schema_error_entity_read(inner_errors.Element)

        parameters = _schema_error_entity_read.parameters
        parameters.Element = AAZStrType()

        _schema.code = cls._schema_error_entity_read.code
        _schema.details = cls._schema_error_entity_read.details
        _schema.extended_code = cls._schema_error_entity_read.extended_code
        _schema.inner_errors = cls._schema_error_entity_read.inner_errors
        _schema.message = cls._schema_error_entity_read.message
        _schema.message_template = cls._schema_error_entity_read.message_template
        _schema.parameters = cls._schema_error_entity_read.parameters
        _schema.target = cls._schema_error_entity_read.target


__all__ = ["Recycle"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
import datetime
from types import SimpleNamespace

# from azure.cli.core.util import CLIError
from azure.cli.core.azclierror import (
    InvalidArgumentValueError,
    RequiredArgumentMissingError,
    MutuallyExclusiveArgumentError,
)

from azure.cli.command_modules.acs.maintenanceconfiguration import aks_maintenanceconfiguration_update_internal
from azure.cli.command_modules.acs.tests.latest.mocks import MockCLI, MockCmd
from azure.cli.core.profiles import ResourceType

class TestAddMaintenanceConfiguration(unittest.TestCase):
    def setUp(self):
        self.cli_ctx = MockCLI()
        self.cmd = MockCmd(self.cli_ctx)
        self.resource_type = ResourceType.MGMT_CONTAINERSERVICE
    def test_add_maintenance_configuration_with_invalid_name(self):
        cmd = SimpleNamespace()
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "something",
        }

        err = ("--config-name must be one of default, aksManagedAutoUpgradeSchedule or aksManagedNodeOSUpgradeSchedule, not something")
        with self.assertRaises(InvalidArgumentValueError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_default_maintenance_configuration_with_schedule_type_and_weekday(self):
        cmd = SimpleNamespace()
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": "Monday",
            "start_hour": 1,
            "schedule_type": "Weekly",
        }

        err = ("--weekday and --start-hour cannot be used together with --schedule-type for default maintenance configuration.")
        with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_default_maintenance_configuration_with_invalid_schedule_type(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Daily",
            "interval_days": 3,
            "interval_weeks": None,
            "interval_months": None,
            "day_of_week": None,
            "day_of_month": None,
            "week_index": None,
        }

        err = ("--schedule-type for default maintenance configuration must be Weekly.")
        with self.assertRaises(InvalidArgumentValueError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_default_maintenance_configuration_rejects_interval_weeks(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": None,
            "interval_weeks": 3,
            "interval_months": None,
            "day_of_week": "Monday",
            "day_of_month": None,
            "week_index": None,
        }

        err = ("--interval-weeks cannot be specified for default maintenance configuration; the interval is always 1 week.")
        with self.assertRaises(InvalidArgumentValueError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_default_maintenance_configuration_rejects_interval_weeks_even_if_1(self):
        """interval_weeks=1 is also rejected; users must simply omit --interval-weeks."""
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": None,
            "interval_weeks": 1,
            "interval_months": None,
            "day_of_week": "Monday",
            "day_of_month": None,
            "week_index": None,
        }

        err = ("--interval-weeks cannot be specified for default maintenance configuration; the interval is always 1 week.")
        with self.assertRaises(InvalidArgumentValueError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_default_maintenance_configuration_rejects_inapplicable_schedule_params(self):
        """interval_days, interval_months, day_of_month, week_index are rejected for default config."""
        for param in ("interval_days", "interval_months", "day_of_month", "week_index"):
            with self.subTest(param=param):
                cmd = MockCmd(self.cli_ctx)
                raw_parameters = {
                    "resource_group_name": "test_rg",
                    "cluster_name": "test_cluster",
                    "config_name": "default",
                    "weekday": None,
                    "start_hour": None,
                    "schedule_type": "Weekly",
                    "interval_days": None,
                    "interval_weeks": None,
                    "interval_months": None,
                    "day_of_week": "Monday",
                    "day_of_month": None,
                    "week_index": None,
                }
                raw_parameters[param] = 1
                expected_flag = "--" + param.replace("_", "-")
                with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
                    aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
                self.assertIn(expected_flag, str(cm.exception))
                self.assertIn("cannot be used for default maintenance configuration", str(cm.exception))

    def test_add_default_maintenance_configuration_rejects_multiple_inapplicable_params(self):
        """Error message names all offending flags when multiple inapplicable params are passed together."""
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": 2,
            "interval_weeks": None,
            "interval_months": None,
            "day_of_week": "Monday",
            "day_of_month": 15,
            "week_index": None,
        }
        with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        msg = str(cm.exception)
        self.assertIn("--interval-days", msg)
        self.assertIn("--day-of-month", msg)
        self.assertIn("cannot be used for default maintenance configuration", msg)
        # flags not passed should not appear in the message
        self.assertNotIn("--interval-months", msg)
        self.assertNotIn("--week-index", msg)

    def test_add_non_default_schedule_with_weekday(self):
        cmd = SimpleNamespace()
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "weekday": "Monday",
        }

        err = ("--weekday and --start-hour are only applicable to default maintenance configuration.")
        with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)
    
    def test_add_daily_schedule_with_missing_options(self):
        cli_ctx = MockCLI()
        cmd = MockCmd(cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "Daily",
        }

        err = ("Please specify --interval-days when using daily schedule.")
        with self.assertRaises(RequiredArgumentMissingError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)
    
    def test_add_daily_schedule_with_invalid_options(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "Daily",
            "interval_days": 3,
            "day_of_week": "Monday",
        }

        err = ("--interval-weeks, --interval-months, --day-of-week, --day-of-month and --week-index cannot be used for Daily schedule.")
        with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_weekly_schedule_with_invalid_options(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "Weekly",
            "day_of_week": "Monday",
            "interval_weeks": 3,
            "week_index": "First",
        }

        err = ("--interval-months, --day-of-month and --week-index cannot be used for Weekly schedule.")
        with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)
    
    def test_add_absolute_monthly_schedule_with_missing_options(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "AbsoluteMonthly",
            "day_of_week": "Monday",
            "interval_months": 3,
        }

        err = ("Please specify --interval-months and --day-of-month when using absolute monthly schedule.")
        with self.assertRaises(RequiredArgumentMissingError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_absolute_monthly_schedule_with_invalid_options(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "AbsoluteMonthly",
            "day_of_month": 15,
            "interval_months": 3,
            "week_index": "First",
        }

        err = ("--interval-days, --interval-weeks, --day-of-week and --week-index cannot be used for AbsoluteMonthly schedule.")
        with self.assertRaises(MutuallyExclusiveArgumentError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)
    
    def test_add_relative_monthly_schedule_with_missing_options(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "RelativeMonthly",
            "day_of_week": "Monday",
            "interval_months": 3,
        }

        err = ("Please specify --interval-months, --day-of-week and --week-index when using relative monthly schedule.")
        with self.assertRaises(RequiredArgumentMissingError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)
    
    def test_add_dedicated_schedule_with_missing_options(self):
        cmd = MockCmd(self.cli_ctx)
        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "aksManagedAutoUpgradeSchedule",
            "schedule_type": "AbsoluteMonthly",
            "day_of_month": 1,
            "interval_months": 3,
            "start_time": "00:00",
        }

        err = ("Please specify --duration for maintenance window.")
        with self.assertRaises(RequiredArgumentMissingError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertEqual(str(cm.exception), err)

    def test_add_default_maintenance_configuration_with_weekly_schedule_type(self):
        cmd = MockCmd(self.cli_ctx)

        class MockMaintenanceConfigClient:
            def create_or_update(self, **kwargs):
                return kwargs.get('parameters')

        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": None,
            "interval_weeks": None,
            "interval_months": None,
            "day_of_week": "Monday",
            "day_of_month": None,
            "week_index": None,
            "start_time": "09:00",
            "duration_hours": 4,
            "utc_offset": None,
            "start_date": None,
        }

        result = aks_maintenanceconfiguration_update_internal(cmd, MockMaintenanceConfigClient(), raw_parameters)

        self.assertIsNotNone(result.maintenance_window)
        self.assertEqual(result.maintenance_window.start_time, "09:00")
        self.assertEqual(result.maintenance_window.duration_hours, 4)
        self.assertIsNotNone(result.maintenance_window.schedule)
        self.assertIsNotNone(result.maintenance_window.schedule.weekly)
        self.assertIsNone(getattr(result, 'time_in_week', None))

    def test_add_default_maintenance_configuration_with_weekly_schedule_type_omits_interval_weeks(self):
        """interval_weeks should default to 1 when omitted for the default config."""
        cmd = MockCmd(self.cli_ctx)

        class MockMaintenanceConfigClient:
            def create_or_update(self, **kwargs):
                return kwargs.get('parameters')

        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": None,
            "interval_weeks": None,
            "interval_months": None,
            "day_of_week": "Monday",
            "day_of_month": None,
            "week_index": None,
            "start_time": "09:00",
            "duration_hours": 4,
            "utc_offset": None,
            "start_date": None,
        }

        result = aks_maintenanceconfiguration_update_internal(cmd, MockMaintenanceConfigClient(), raw_parameters)

        self.assertIsNotNone(result.maintenance_window)
        self.assertIsNotNone(result.maintenance_window.schedule)
        self.assertIsNotNone(result.maintenance_window.schedule.weekly)
        self.assertEqual(result.maintenance_window.schedule.weekly.interval_weeks, 1)
        self.assertIsNone(raw_parameters["interval_weeks"])
        self.assertIsNone(getattr(result, 'time_in_week', None))

    def test_add_default_maintenance_configuration_with_utc_offset_and_start_date(self):
        """utc_offset and start_date must be passed through to the constructed maintenance_window."""
        cmd = MockCmd(self.cli_ctx)

        class MockMaintenanceConfigClient:
            def create_or_update(self, **kwargs):
                return kwargs.get('parameters')

        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": None,
            "interval_weeks": None,
            "interval_months": None,
            "day_of_week": "Monday",
            "day_of_month": None,
            "week_index": None,
            "start_time": "09:00",
            "duration_hours": 4,
            "utc_offset": "+05:30",
            "start_date": "2026-01-15",
        }

        result = aks_maintenanceconfiguration_update_internal(cmd, MockMaintenanceConfigClient(), raw_parameters)

        self.assertIsNotNone(result.maintenance_window)
        self.assertEqual(result.maintenance_window.utc_offset, "+05:30")
        self.assertEqual(result.maintenance_window.start_date, datetime.date(2026, 1, 15))

    def test_add_default_maintenance_configuration_requires_day_of_week(self):
        """--schedule-type Weekly without --day-of-week must raise a clear error for default config."""
        cmd = MockCmd(self.cli_ctx)

        raw_parameters = {
            "resource_group_name": "test_rg",
            "cluster_name": "test_cluster",
            "config_name": "default",
            "weekday": None,
            "start_hour": None,
            "schedule_type": "Weekly",
            "interval_days": None,
            "interval_weeks": None,
            "interval_months": None,
            "day_of_week": None,
            "day_of_month": None,
            "week_index": None,
            "start_time": "09:00",
            "duration_hours": 4,
            "utc_offset": None,
            "start_date": None,
        }

        with self.assertRaises(RequiredArgumentMissingError) as cm:
            aks_maintenanceconfiguration_update_internal(cmd, None, raw_parameters)
        self.assertIn("--day-of-week", str(cm.exception))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""
Unit tests for the Compose → Sitecontainers conversion helpers in custom.py.

These tests validate the parsing functions without requiring Azure connectivity.
Run with: python -m pytest <this_file> -v
"""

import os
import unittest
from base64 import b64encode
from unittest.mock import MagicMock, patch, call

import yaml


# Import the helpers under test
from azure.cli.command_modules.appservice.custom import (
    _parse_compose_entrypoint_or_command,
    _merge_entrypoint_command,
    _parse_compose_environment,
    _parse_compose_ports,
    _parse_compose_volumes,
    _make_bind_mount,
    _make_named_volume_mount,
    _sanitize_container_name,
    _convert_compose_to_sitecontainers,
    _COMPOSE_WEBAPP_STORAGE_HOME,
)

SAMPLES_DIR = os.path.dirname(__file__)


def _load_sample(filename):
    """Load and parse a sample compose YAML from the test directory."""
    path = os.path.join(SAMPLES_DIR, filename)
    with open(path, 'r', encoding='utf-8') as f:
        return yaml.safe_load(f)


def _b64_encode_file(filename):
    """Return the base64-encoded contents of a sample file (simulating linuxFxVersion)."""
    path = os.path.join(SAMPLES_DIR, filename)
    with open(path, 'rb') as f:
        return b64encode(f.read()).decode('utf-8')


# ---------------------------------------------------------------------------
# _sanitize_container_name
# ---------------------------------------------------------------------------
class TestSanitizeContainerName(unittest.TestCase):
    def test_simple_name(self):
        self.assertEqual(_sanitize_container_name("web"), "web")

    def test_underscores_replaced(self):
        self.assertEqual(_sanitize_container_name("my_web_app"), "my-web-app")

    def test_dots_replaced(self):
        self.assertEqual(_sanitize_container_name("background.worker"), "background-worker")

    def test_uppercase_lowered(self):
        self.assertEqual(_sanitize_container_name("UPPERCASE_SVC"), "uppercase-svc")

    def test_already_valid(self):
        self.assertEqual(_sanitize_container_name("data-processor"), "data-processor")

    def test_consecutive_specials_collapsed(self):
        self.assertEqual(_sanitize_container_name("a__b..c"), "a-b-c")

    def test_empty_string(self):
        self.assertEqual(_sanitize_container_name(""), "container")

    def test_leading_trailing_hyphens_stripped(self):
        self.assertEqual(_sanitize_container_name("_leading_"), "leading")


# ---------------------------------------------------------------------------
# _parse_compose_entrypoint_or_command
# ---------------------------------------------------------------------------
class TestParseEntrypointOrCommand(unittest.TestCase):
    def test_none_returns_empty(self):
        self.assertEqual(_parse_compose_entrypoint_or_command(None), [])

    def test_string_split_on_whitespace(self):
        result = _parse_compose_entrypoint_or_command("gunicorn --bind 0.0.0.0:5000 app:app")
        self.assertEqual(result, ["gunicorn", "--bind", "0.0.0.0:5000", "app:app"])

    def test_list_preserved(self):
        result = _parse_compose_entrypoint_or_command(["celery", "-A", "tasks"])
        self.assertEqual(result, ["celery", "-A", "tasks"])

    def test_single_word_string(self):
        result = _parse_compose_entrypoint_or_command("python")
        self.assertEqual(result, ["python"])

    def test_entrypoint_empty_string(self):
        result = _parse_compose_entrypoint_or_command("")
        self.assertEqual(result, [])


# ---------------------------------------------------------------------------
# _merge_entrypoint_command
# ---------------------------------------------------------------------------
class TestMergeEntrypointCommand(unittest.TestCase):
    def test_both_empty_returns_none(self):
        self.assertIsNone(_merge_entrypoint_command([], []))

    def test_only_entrypoint(self):
        result = _merge_entrypoint_command(["python", "app.py"], [])
        self.assertEqual(result, "python app.py")

    def test_only_command(self):
        result = _merge_entrypoint_command([], ["--workers", "4"])
        self.assertEqual(result, "--workers 4")

    def test_both_merged(self):
        result = _merge_entrypoint_command(["gunicorn"], ["--bind", "0.0.0.0:5000", "app:app"])
        self.assertEqual(result, "gunicorn --bind 0.0.0.0:5000 app:app")


# ---------------------------------------------------------------------------
# _parse_compose_environment
# ---------------------------------------------------------------------------
class TestParseComposeEnvironment(unittest.TestCase):
    def test_env_none_returns_empty(self):
        self.assertEqual(_parse_compose_environment(None), {})

    def test_mapping_format(self):
        env = {"NODE_ENV": "production", "PORT": 3000, "EMPTY": None}
        result = _parse_compose_environment(env)
        self.assertEqual(result, {"NODE_ENV": "production", "PORT": "3000", "EMPTY": ""})

    def test_sequence_with_values(self):
        env = ["KEY1=value1", "KEY2=value2"]
        result = _parse_compose_environment(env)
        self.assertEqual(result, {"KEY1": "value1", "KEY2": "value2"})

    def test_sequence_value_with_equals(self):
        """Value itself contains '=' characters."""
        env = ["CONNECTION=Server=localhost;Database=mydb"]
        result = _parse_compose_environment(env)
        self.assertEqual(result, {"CONNECTION": "Server=localhost;Database=mydb"})

    def test_sequence_name_only(self):
        """No '=' means value-less reference to an app setting."""
        env = ["EXISTING_SETTING"]
        result = _parse_compose_environment(env)
        self.assertEqual(result, {"EXISTING_SETTING": ""})

    def test_sequence_empty_value(self):
        """Trailing '=' means empty value."""
        env = ["KEY="]
        result = _parse_compose_environment(env)
        self.assertEqual(result, {"KEY": ""})


# ---------------------------------------------------------------------------
# _parse_compose_ports
# ---------------------------------------------------------------------------
class TestParseComposePorts(unittest.TestCase):
    def test_ports_none_returns_empty(self):
        self.assertEqual(_parse_compose_ports(None), [])

    def test_host_container_mapping(self):
        result = _parse_compose_ports(["8080:80", "3306:3306"])
        self.assertEqual(result, [(8080, 80), (3306, 3306)])

    def test_single_port_no_host(self):
        result = _parse_compose_ports(["80"])
        self.assertEqual(result, [(None, 80)])

    def test_invalid_port_skipped(self):
        result = _parse_compose_ports(["abc:xyz", "8080:80"])
        self.assertEqual(result, [(8080, 80)])


# ---------------------------------------------------------------------------
# _parse_compose_volumes
# ---------------------------------------------------------------------------
class TestParseComposeVolumes(unittest.TestCase):
    def test_volumes_none_returns_empty(self):
        mounts, warnings = _parse_compose_volumes(None, {})
        self.assertEqual(mounts, [])
        self.assertEqual(warnings, [])

    def test_bind_mount_short_syntax(self):
        volumes = ["${WEBAPP_STORAGE_HOME}/site/wwwroot:/var/www/html"]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 1)
        self.assertEqual(mounts[0]["volume_sub_path"], "/home/site/wwwroot")
        self.assertEqual(mounts[0]["container_mount_path"], "/var/www/html")
        self.assertFalse(mounts[0]["read_only"])

    def test_bind_mount_root_home(self):
        """${WEBAPP_STORAGE_HOME} alone (no subpath) maps to /home."""
        volumes = ["${WEBAPP_STORAGE_HOME}:/mnt/home"]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 1)
        self.assertEqual(mounts[0]["volume_sub_path"], "/home")
        self.assertEqual(mounts[0]["container_mount_path"], "/mnt/home")

    def test_bind_mount_with_ro(self):
        volumes = ["${WEBAPP_STORAGE_HOME}/config:/etc/config:ro"]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 1)
        self.assertTrue(mounts[0]["read_only"])

    def test_unsupported_host_path(self):
        volumes = ["/var/data:/app/data"]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 0)
        self.assertTrue(any("UNSUPPORTED" in w for w in warnings))

    def test_relative_path_unsupported(self):
        volumes = ["./config:/app/config"]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 0)
        self.assertTrue(any("UNSUPPORTED" in w for w in warnings))

    def test_named_volume(self):
        volumes = ["mydata:/app/data"]
        mounts, warnings = _parse_compose_volumes(volumes, {"mydata": None})
        self.assertEqual(len(mounts), 1)
        self.assertEqual(mounts[0]["volume_sub_path"], "/compose/volumes/mydata")
        self.assertEqual(mounts[0]["container_mount_path"], "/app/data")
        self.assertTrue(any("Named volume" in w for w in warnings))

    def test_long_syntax_bind(self):
        volumes = [{"type": "bind", "source": "${WEBAPP_STORAGE_HOME}/site/wwwroot", "target": "/app"}]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 1)
        self.assertEqual(mounts[0]["volume_sub_path"], "/home/site/wwwroot")
        self.assertEqual(mounts[0]["container_mount_path"], "/app")

    def test_long_syntax_bind_read_only(self):
        volumes = [{
            "type": "bind",
            "source": "${WEBAPP_STORAGE_HOME}/conf",
            "target": "/etc/conf",
            "read_only": True,
        }]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 1)
        self.assertTrue(mounts[0]["read_only"])

    def test_long_syntax_volume(self):
        volumes = [{"type": "volume", "source": "cache-vol", "target": "/tmp/cache"}]
        mounts, warnings = _parse_compose_volumes(volumes, {"cache-vol": None})
        self.assertEqual(len(mounts), 1)
        self.assertEqual(mounts[0]["volume_sub_path"], "/compose/volumes/cache-vol")

    def test_mixed_supported_and_unsupported(self):
        """Mix of valid bind mount, invalid host path, and named volume."""
        volumes = [
            "${WEBAPP_STORAGE_HOME}/site/wwwroot:/app/public",
            "/var/data:/app/data",
            "${WEBAPP_STORAGE_HOME}/logs:/app/logs",
        ]
        mounts, warnings = _parse_compose_volumes(volumes, {})
        self.assertEqual(len(mounts), 2)  # Two valid bind mounts
        self.assertTrue(any("UNSUPPORTED" in w for w in warnings))


# ---------------------------------------------------------------------------
# _make_bind_mount
# ---------------------------------------------------------------------------
class TestMakeBindMount(unittest.TestCase):
    def test_standard_path(self):
        warnings = []
        result = _make_bind_mount("${WEBAPP_STORAGE_HOME}/site/wwwroot", "/var/www/html", False, warnings)
        self.assertIsNotNone(result)
        self.assertEqual(result["volume_sub_path"], "/home/site/wwwroot")
        self.assertEqual(result["container_mount_path"], "/var/www/html")
        self.assertEqual(len(warnings), 0)

    def test_root_only(self):
        warnings = []
        result = _make_bind_mount("${WEBAPP_STORAGE_HOME}", "/mnt", False, warnings)
        self.assertEqual(result["volume_sub_path"], "/home")

    def test_non_matching_source_returns_none(self):
        warnings = []
        result = _make_bind_mount("/some/host/path", "/app", False, warnings)
        self.assertIsNone(result)
        self.assertEqual(len(warnings), 1)

    def test_nested_path(self):
        warnings = []
        result = _make_bind_mount(
            "${WEBAPP_STORAGE_HOME}/deep/nested/path/here", "/container/path", False, warnings
        )
        self.assertEqual(result["volume_sub_path"], "/home/deep/nested/path/here")


# ---------------------------------------------------------------------------
# Integration-style: load sample YAML and validate structure
# ---------------------------------------------------------------------------
class TestSampleYamlParsing(unittest.TestCase):
    """Validate that sample compose YAMLs parse correctly with yaml.safe_load."""

    def _load(self, filename):
        return _load_sample(filename)

    def test_basic_has_two_services(self):
        compose = self._load("compose-convert-basic.yml")
        self.assertIn("services", compose)
        self.assertEqual(len(compose["services"]), 2)
        self.assertIn("web", compose["services"])
        self.assertIn("redis", compose["services"])

    def test_env_mapping_format(self):
        compose = self._load("compose-convert-env-mapping.yml")
        api_env = compose["services"]["api"]["environment"]
        self.assertIsInstance(api_env, dict)
        self.assertEqual(api_env["NODE_ENV"], "production")

    def test_env_sequence_format(self):
        compose = self._load("compose-convert-env-sequence.yml")
        app_env = compose["services"]["app"]["environment"]
        self.assertIsInstance(app_env, list)
        self.assertIn("REDIS_URL=redis://localhost:6379", app_env)

    def test_volumes_bind_mount(self):
        compose = self._load("compose-convert-volumes-bind.yml")
        wp_vols = compose["services"]["wordpress"]["volumes"]
        self.assertEqual(len(wp_vols), 2)
        self.assertTrue(wp_vols[0].startswith("${WEBAPP_STORAGE_HOME}"))

    def test_volumes_named(self):
        compose = self._load("compose-convert-volumes-named.yml")
        self.assertIn("volumes", compose)
        self.assertIn("app-data", compose["volumes"])

    def test_volumes_long_syntax(self):
        compose = self._load("compose-convert-volumes-long.yml")
        web_vols = compose["services"]["web"]["volumes"]
        self.assertIsInstance(web_vols[0], dict)
        self.assertEqual(web_vols[0]["type"], "bind")

    def test_entrypoint_command(self):
        compose = self._load("compose-convert-entrypoint-command.yml")
        web = compose["services"]["web"]
        self.assertEqual(web["entrypoint"], "gunicorn")
        self.assertEqual(web["command"], "--bind 0.0.0.0:5000 app:app --workers 4")
        worker = compose["services"]["worker"]
        self.assertIsInstance(worker["entrypoint"], list)
        self.assertIsInstance(worker["command"], list)

    def test_port_conflict_file_parses(self):
        compose = self._load("compose-convert-port-conflict.yml")
        # Both services have port 8080 as container port
        fe_ports = compose["services"]["frontend"]["ports"]
        be_ports = compose["services"]["backend"]["ports"]
        self.assertEqual(fe_ports, ["80:8080"])
        self.assertEqual(be_ports, ["8080:8080"])

    def test_full_scenario_service_count(self):
        compose = self._load("compose-convert-full.yml")
        self.assertEqual(len(compose["services"]), 4)

    def test_underscore_names_parse(self):
        compose = self._load("compose-convert-underscore-names.yml")
        services = list(compose["services"].keys())
        self.assertIn("my_web_app", services)
        self.assertIn("background.worker", services)
        self.assertIn("UPPERCASE_SVC", services)

    def test_no_ports_file_parses(self):
        compose = self._load("compose-convert-no-ports.yml")
        # Neither service has ports
        for svc in compose["services"].values():
            self.assertNotIn("ports", svc)

    def test_b64_encode_roundtrip(self):
        """Verify base64 encode/decode preserves the YAML."""
        from base64 import b64decode
        b64 = _b64_encode_file("compose-convert-basic.yml")
        decoded = b64decode(b64.encode('utf-8')).decode('utf-8')
        compose = yaml.safe_load(decoded)
        self.assertIn("services", compose)
        self.assertEqual(len(compose["services"]), 2)


# ---------------------------------------------------------------------------
# End-to-end parsing of volumes from sample files
# ---------------------------------------------------------------------------
class TestSampleVolumeParsing(unittest.TestCase):
    """Parse volumes from sample YAMLs through the actual helper functions."""

    def test_bind_mount_file(self):
        compose = _load_sample("compose-convert-volumes-bind.yml")
        wp_vols = compose["services"]["wordpress"]["volumes"]
        mounts, warnings = _parse_compose_volumes(wp_vols, {})
        self.assertEqual(len(mounts), 2)
        self.assertEqual(mounts[0]["volume_sub_path"], "/home/site/wwwroot")
        self.assertEqual(mounts[0]["container_mount_path"], "/var/www/html")
        self.assertEqual(mounts[1]["volume_sub_path"], "/home/wordpress/uploads")

    def test_named_volume_file(self):
        compose = _load_sample("compose-convert-volumes-named.yml")
        top_vols = compose.get("volumes", {})
        app_vols = compose["services"]["app"]["volumes"]
        mounts, warnings = _parse_compose_volumes(app_vols, top_vols)
        self.assertEqual(len(mounts), 2)
        self.assertEqual(mounts[0]["volume_sub_path"], "/compose/volumes/app-data")
        self.assertTrue(any("Named volume" in w for w in warnings))

    def test_long_syntax_file(self):
        compose = _load_sample("compose-convert-volumes-long.yml")
        web_vols = compose["services"]["web"]["volumes"]
        mounts, warnings = _parse_compose_volumes(web_vols, {})
        self.assertEqual(len(mounts), 2)
        self.assertEqual(mounts[0]["volume_sub_path"], "/home/site/wwwroot")
        self.assertEqual(mounts[0]["container_mount_path"], "/usr/share/nginx/html")
        self.assertTrue(mounts[1]["read_only"])  # second mount has read_only: true

    def test_unsupported_bind_file(self):
        compose = _load_sample("compose-convert-unsupported-bind.yml")
        app_vols = compose["services"]["app"]["volumes"]
        mounts, warnings = _parse_compose_volumes(app_vols, {})
        # 2 valid ${WEBAPP_STORAGE_HOME} mounts, 2 unsupported
        self.assertEqual(len(mounts), 2)
        unsupported_warnings = [w for w in warnings if "UNSUPPORTED" in w]
        self.assertEqual(len(unsupported_warnings), 2)

    def test_full_scenario_volumes(self):
        compose = _load_sample("compose-convert-full.yml")
        # WordPress has short + long syntax volumes
        wp_vols = compose["services"]["wordpress"]["volumes"]
        mounts, warnings = _parse_compose_volumes(wp_vols, {})
        self.assertEqual(len(mounts), 2)
        paths = [m["volume_sub_path"] for m in mounts]
        self.assertIn("/home/site/wwwroot", paths)
        self.assertIn("/home/wordpress/uploads", paths)


# ---------------------------------------------------------------------------
# End-to-end parsing of entrypoint/command from sample files
# ---------------------------------------------------------------------------
class TestSampleEntrypointParsing(unittest.TestCase):
    def test_string_entrypoint_and_command(self):
        compose = _load_sample("compose-convert-entrypoint-command.yml")
        web = compose["services"]["web"]
        ep = _parse_compose_entrypoint_or_command(web.get("entrypoint"))
        cmd = _parse_compose_entrypoint_or_command(web.get("command"))
        merged = _merge_entrypoint_command(ep, cmd)
        self.assertEqual(merged, "gunicorn --bind 0.0.0.0:5000 app:app --workers 4")

    def test_list_entrypoint_and_command(self):
        compose = _load_sample("compose-convert-entrypoint-command.yml")
        worker = compose["services"]["worker"]
        ep = _parse_compose_entrypoint_or_command(worker.get("entrypoint"))
        cmd = _parse_compose_entrypoint_or_command(worker.get("command"))
        merged = _merge_entrypoint_command(ep, cmd)
        self.assertEqual(merged, "celery -A tasks worker --loglevel=info --concurrency=2")

    def test_command_only(self):
        compose = _load_sample("compose-convert-entrypoint-command.yml")
        scheduler = compose["services"]["scheduler"]
        ep = _parse_compose_entrypoint_or_command(scheduler.get("entrypoint"))
        cmd = _parse_compose_entrypoint_or_command(scheduler.get("command"))
        merged = _merge_entrypoint_command(ep, cmd)
        self.assertEqual(merged, "celery -A tasks beat --loglevel=info")

    def test_entrypoint_only(self):
        compose = _load_sample("compose-convert-entrypoint-command.yml")
        sidecar = compose["services"]["sidecar"]
        ep = _parse_compose_entrypoint_or_command(sidecar.get("entrypoint"))
        cmd = _parse_compose_entrypoint_or_command(sidecar.get("command"))
        merged = _merge_entrypoint_command(ep, cmd)
        self.assertEqual(merged, "/usr/local/bin/monitor --port 9090")


# ---------------------------------------------------------------------------
# End-to-end parsing of environment variables from sample files
# ---------------------------------------------------------------------------
class TestSampleEnvParsing(unittest.TestCase):
    def test_env_mapping_format_from_file(self):
        compose = _load_sample("compose-convert-env-mapping.yml")
        env = _parse_compose_environment(compose["services"]["api"]["environment"])
        self.assertEqual(env["NODE_ENV"], "production")
        self.assertEqual(env["DB_HOST"], "localhost")
        self.assertEqual(env["DB_PORT"], "5432")

    def test_sequence_format(self):
        compose = _load_sample("compose-convert-env-sequence.yml")
        env = _parse_compose_environment(compose["services"]["app"]["environment"])
        self.assertEqual(env["REDIS_URL"], "redis://localhost:6379")
        self.assertEqual(env["EXISTING_SETTING"], "")  # name-only reference

    def test_multiline_value(self):
        compose = _load_sample("compose-convert-full.yml")
        env = _parse_compose_environment(compose["services"]["wordpress"]["environment"])
        # WORDPRESS_CONFIG_EXTRA has multiline YAML value
        self.assertIn("WP_REDIS_HOST", env["WORDPRESS_CONFIG_EXTRA"])


# ---------------------------------------------------------------------------
# Port parsing from sample files
# ---------------------------------------------------------------------------
class TestSamplePortParsing(unittest.TestCase):
    def test_basic_ports(self):
        compose = _load_sample("compose-convert-basic.yml")
        ports = _parse_compose_ports(compose["services"]["web"].get("ports"))
        self.assertEqual(ports, [(8080, 8080)])

    def test_port_conflict_different_host_same_container(self):
        compose = _load_sample("compose-convert-port-conflict.yml")
        fe_ports = _parse_compose_ports(compose["services"]["frontend"].get("ports"))
        be_ports = _parse_compose_ports(compose["services"]["backend"].get("ports"))
        # Both have container port 8080
        self.assertEqual(fe_ports[0][1], 8080)
        self.assertEqual(be_ports[0][1], 8080)

    def test_multi_port(self):
        compose = _load_sample("compose-convert-multi-port.yml")
        ports = _parse_compose_ports(compose["services"]["app"].get("ports"))
        self.assertEqual(len(ports), 3)
        self.assertEqual(ports[0], (80, 80))
        self.assertEqual(ports[1], (443, 443))
        self.assertEqual(ports[2], (8080, 8080))

    def test_no_ports(self):
        compose = _load_sample("compose-convert-no-ports.yml")
        ports = _parse_compose_ports(compose["services"]["processor"].get("ports"))
        self.assertEqual(ports, [])


# ---------------------------------------------------------------------------
# _make_named_volume_mount (direct tests)
# ---------------------------------------------------------------------------
class TestMakeNamedVolumeMount(unittest.TestCase):
    def test_basic_named_volume(self):
        warnings = []
        result = _make_named_volume_mount("mydata", "/app/data", False, {}, warnings)
        self.assertIsNotNone(result)
        self.assertEqual(result["volume_sub_path"], "/compose/volumes/mydata")
        self.assertEqual(result["container_mount_path"], "/app/data")
        self.assertFalse(result["read_only"])
        self.assertTrue(any("Named volume" in w for w in warnings))
        self.assertTrue(any("ephemeral" in w.lower() or "LOCAL" in w for w in warnings))

    def test_named_volume_read_only(self):
        warnings = []
        result = _make_named_volume_mount("cache", "/tmp/cache", True, {}, warnings)
        self.assertTrue(result["read_only"])

    def test_named_volume_with_top_level(self):
        warnings = []
        result = _make_named_volume_mount("pgdata", "/var/lib/pg", False, {"pgdata": None}, warnings)
        self.assertEqual(result["volume_sub_path"], "/compose/volumes/pgdata")

    def test_named_volume_persistence_warning(self):
        warnings = []
        _make_named_volume_mount("data", "/data", False, {}, warnings)
        self.assertTrue(any("persist" in w.lower() or "restart" in w.lower() for w in warnings))


# ---------------------------------------------------------------------------
# Helpers to build mocked objects for orchestration tests
# ---------------------------------------------------------------------------
def _make_compose_b64(compose_dict):
    """Encode a compose dict as COMPOSE|<base64> linuxFxVersion."""
    yaml_str = yaml.dump(compose_dict, default_flow_style=False)
    b64 = b64encode(yaml_str.encode('utf-8')).decode('utf-8')
    return f"COMPOSE|{b64}"


def _make_mock_site_config(**kwargs):
    """Create a mock site_config object."""
    config = MagicMock()
    config.acr_use_managed_identity_creds = kwargs.get("acr_use_managed_identity_creds", None)
    config.acr_user_managed_identity_id = kwargs.get("acr_user_managed_identity_id", None)
    return config


def _make_mock_cmd(existing_app_settings=None):
    """Create a mock cmd object with CLI context for ARM calls."""
    cmd = MagicMock()
    cmd.cli_ctx = MagicMock()
    cmd.cli_ctx.cloud.endpoints.resource_manager = "https://management.azure.com"

    # Mock send_raw_request to return existing app settings
    settings = existing_app_settings or {}
    response = MagicMock()
    response.json.return_value = {"properties": settings}
    return cmd, response


# Patch targets in the module under test
_CUSTOM_MOD = "azure.cli.command_modules.appservice.custom"
_GET_SUB_ID = "azure.cli.core.commands.client_factory.get_subscription_id"


# ---------------------------------------------------------------------------
# Orchestration: Main container detection
# ---------------------------------------------------------------------------
class TestMainContainerDetection(unittest.TestCase):
    """Test main container auto-detection logic in _convert_compose_to_sitecontainers."""

    def _run_conversion(self, compose_dict, main_container_name=None, existing_settings=None):
        """Run the conversion with mocked ARM calls and return created sitecontainers."""
        cmd, raw_response = _make_mock_cmd(existing_settings)
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose_dict)
        created = []

        def track_create(cmd, name, rg, container_name, sitecontainer, slot):
            created.append({
                "container_name": container_name,
                "is_main": sitecontainer.is_main,
                "image": sitecontainer.image,
                "target_port": sitecontainer.target_port,
            })
            return MagicMock()

        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="00000000-0000-0000-0000-000000000000"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal", side_effect=track_create), \
             patch(f"{_CUSTOM_MOD}.update_app_settings"), \
             patch(f"{_CUSTOM_MOD}.update_site_configs"), \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            _convert_compose_to_sitecontainers(
                cmd, "testapp", "testrg", None, site_config, linux_fx, main_container_name
            )
        return created

    def test_single_service_with_port_is_main(self):
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["80:80"]},
                "redis": {"image": "redis:alpine"},
            }
        }
        created = self._run_conversion(compose)
        self.assertEqual(len(created), 2)
        web = next(c for c in created if c["container_name"] == "web")
        redis = next(c for c in created if c["container_name"] == "redis")
        self.assertTrue(web["is_main"])
        self.assertFalse(redis["is_main"])

    def test_multiple_ports_first_is_main(self):
        # yaml.dump sorts keys alphabetically, so 'api' < 'web' means 'api' is first
        compose = {
            "version": "3",
            "services": {
                "api": {"image": "nginx:alpine", "ports": ["80:80"]},
                "web": {"image": "node:20", "ports": ["8080:8080"]},
            }
        }
        created = self._run_conversion(compose)
        api = next(c for c in created if c["container_name"] == "api")
        web = next(c for c in created if c["container_name"] == "web")
        self.assertTrue(api["is_main"])
        self.assertFalse(web["is_main"])

    def test_no_ports_first_service_is_main(self):
        # yaml.dump sorts keys alphabetically, so 'alpha' < 'beta' means 'alpha' is first
        compose = {
            "version": "3",
            "services": {
                "alpha": {"image": "busybox:latest"},
                "beta": {"image": "busybox:latest"},
            }
        }
        created = self._run_conversion(compose)
        alpha = next(c for c in created if c["container_name"] == "alpha")
        beta = next(c for c in created if c["container_name"] == "beta")
        self.assertTrue(alpha["is_main"])
        self.assertFalse(beta["is_main"])

    def test_explicit_main_container_name_by_service(self):
        compose = {
            "version": "3",
            "services": {
                "frontend": {"image": "nginx:alpine", "ports": ["80:80"]},
                "backend": {"image": "node:20", "ports": ["8080:8080"]},
            }
        }
        created = self._run_conversion(compose, main_container_name="backend")
        frontend = next(c for c in created if c["container_name"] == "frontend")
        backend = next(c for c in created if c["container_name"] == "backend")
        self.assertFalse(frontend["is_main"])
        self.assertTrue(backend["is_main"])

    def test_explicit_main_container_name_not_found_raises(self):
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["80:80"]},
            }
        }
        from azure.cli.core.azclierror import ValidationError
        with self.assertRaises(ValidationError):
            self._run_conversion(compose, main_container_name="nonexistent")


# ---------------------------------------------------------------------------
# Orchestration: Container name collision
# ---------------------------------------------------------------------------
class TestContainerNameCollision(unittest.TestCase):

    def test_collision_raises_validation_error(self):
        """Services 'my_web_app' and 'my.web.app' both sanitize to 'my-web-app'."""
        compose = {
            "version": "3",
            "services": {
                "my_web_app": {"image": "nginx:alpine", "ports": ["80:80"]},
                "my.web.app": {"image": "httpd:alpine"},
            }
        }
        cmd, raw_response = _make_mock_cmd()
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose)

        from azure.cli.core.azclierror import ValidationError
        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            with self.assertRaises(ValidationError) as ctx:
                _convert_compose_to_sitecontainers(
                    cmd, "testapp", "testrg", None, site_config, linux_fx
                )
            self.assertIn("collision", str(ctx.exception).lower())


# ---------------------------------------------------------------------------
# Orchestration: Environment variable app setting naming
# ---------------------------------------------------------------------------
class TestEnvVarAppSettingNaming(unittest.TestCase):

    def _run_and_capture_settings(self, compose_dict, existing_settings=None):
        """Run conversion and capture the app settings that would be created."""
        cmd, raw_response = _make_mock_cmd(existing_settings)
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose_dict)
        captured_settings = {}

        def capture_update_settings(cmd, rg, name, settings_list, slot=None):
            for s in settings_list:
                key, val = s.split("=", 1)
                captured_settings[key] = val

        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal", return_value=MagicMock()), \
             patch(f"{_CUSTOM_MOD}.update_app_settings", side_effect=capture_update_settings), \
             patch(f"{_CUSTOM_MOD}.update_site_configs"), \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            _convert_compose_to_sitecontainers(
                cmd, "testapp", "testrg", None, site_config, linux_fx
            )
        return captured_settings

    def test_env_vars_create_compose_prefixed_settings(self):
        compose = {
            "version": "3",
            "services": {
                "web": {
                    "image": "nginx:alpine",
                    "ports": ["80:80"],
                    "environment": {"MY_VAR": "hello", "OTHER": "world"},
                }
            }
        }
        settings = self._run_and_capture_settings(compose)
        self.assertIn("COMPOSE_WEB_MY_VAR", settings)
        self.assertEqual(settings["COMPOSE_WEB_MY_VAR"], "hello")
        self.assertIn("COMPOSE_WEB_OTHER", settings)
        self.assertEqual(settings["COMPOSE_WEB_OTHER"], "world")

    def test_env_var_service_name_sanitized_in_key(self):
        """Service with underscores should have underscores in setting key (not hyphens)."""
        compose = {
            "version": "3",
            "services": {
                "my-api": {
                    "image": "node:20",
                    "ports": ["8080:8080"],
                    "environment": {"PORT": "8080"},
                }
            }
        }
        settings = self._run_and_capture_settings(compose)
        # _sanitize_container_name("my-api") -> "my-api", then upper + replace - with _
        self.assertIn("COMPOSE_MY_API_PORT", settings)

    def test_valueless_env_var_references_existing_setting(self):
        """Env var with no value should reference existing app setting directly."""
        compose = {
            "version": "3",
            "services": {
                "web": {
                    "image": "nginx:alpine",
                    "ports": ["80:80"],
                    "environment": ["EXISTING_KEY"],
                }
            }
        }
        # EXISTING_KEY already exists as an app setting
        settings = self._run_and_capture_settings(compose, existing_settings={"EXISTING_KEY": "some-value"})
        # Should NOT create a COMPOSE_ prefixed setting for it
        self.assertNotIn("COMPOSE_WEB_EXISTING_KEY", settings)

    def test_valueless_env_var_no_existing_creates_empty(self):
        """Env var with no value and no existing setting creates empty COMPOSE_ key."""
        compose = {
            "version": "3",
            "services": {
                "web": {
                    "image": "nginx:alpine",
                    "ports": ["80:80"],
                    "environment": ["MISSING_KEY"],
                }
            }
        }
        settings = self._run_and_capture_settings(compose, existing_settings={})
        self.assertIn("COMPOSE_WEB_MISSING_KEY", settings)
        self.assertEqual(settings["COMPOSE_WEB_MISSING_KEY"], "")


# ---------------------------------------------------------------------------
# Orchestration: Port conflict warnings
# ---------------------------------------------------------------------------
class TestPortConflictWarnings(unittest.TestCase):

    def _run_and_capture_warnings(self, compose_dict):
        """Run conversion and capture logger warnings."""
        cmd, raw_response = _make_mock_cmd()
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose_dict)
        warnings = []

        def capture_warning(msg, *args):
            warnings.append(msg % args if args else msg)

        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal", return_value=MagicMock()), \
             patch(f"{_CUSTOM_MOD}.update_app_settings"), \
             patch(f"{_CUSTOM_MOD}.update_site_configs"), \
             patch(f"{_CUSTOM_MOD}.logger") as mock_logger, \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            mock_logger.warning = capture_warning
            _convert_compose_to_sitecontainers(
                cmd, "testapp", "testrg", None, site_config, linux_fx
            )
        return warnings

    def test_port_conflict_critical_warning(self):
        compose = {
            "version": "3",
            "services": {
                "frontend": {"image": "nginx:alpine", "ports": ["80:80"]},
                "backend": {"image": "httpd:alpine", "ports": ["8080:80"]},
            }
        }
        warnings = self._run_and_capture_warnings(compose)
        critical = [w for w in warnings if "CRITICAL" in w and "80" in w]
        self.assertTrue(len(critical) > 0, "Expected a CRITICAL port conflict warning for port 80")

    def test_host_container_port_mismatch_warning(self):
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["8080:3000"]},
            }
        }
        warnings = self._run_and_capture_warnings(compose)
        mismatch = [w for w in warnings if "Host port" in w and "8080" in w and "3000" in w]
        self.assertTrue(len(mismatch) > 0, "Expected host/container port mismatch warning")

    def test_multiple_ports_warning(self):
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["80:80", "443:443"]},
            }
        }
        warnings = self._run_and_capture_warnings(compose)
        multi = [w for w in warnings if "Multiple port mappings" in w]
        self.assertTrue(len(multi) > 0, "Expected multiple port mappings warning")

    def test_unsupported_keys_warning(self):
        compose = {
            "version": "3",
            "services": {
                "web": {
                    "image": "nginx:alpine",
                    "ports": ["80:80"],
                    "depends_on": ["redis"],
                    "healthcheck": {"test": "curl http://localhost"},
                },
                "redis": {"image": "redis:alpine"},
            }
        }
        warnings = self._run_and_capture_warnings(compose)
        dep_warnings = [w for w in warnings if "depends_on" in w]
        hc_warnings = [w for w in warnings if "healthcheck" in w]
        self.assertTrue(len(dep_warnings) > 0, "Expected depends_on warning")
        self.assertTrue(len(hc_warnings) > 0, "Expected healthcheck warning")

    def test_networking_notice_always_shown(self):
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["80:80"]},
            }
        }
        warnings = self._run_and_capture_warnings(compose)
        net = [w for w in warnings if "localhost" in w and "network namespace" in w]
        self.assertTrue(len(net) > 0, "Expected networking change notice")


# ---------------------------------------------------------------------------
# Orchestration: Rollback on failure
# ---------------------------------------------------------------------------
class TestRollbackOnFailure(unittest.TestCase):

    def test_rollback_deletes_created_containers(self):
        """If creating the 2nd container fails, the 1st should be rolled back."""
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["80:80"]},
                "sidecar": {"image": "redis:alpine"},
            }
        }
        cmd, raw_response = _make_mock_cmd()
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose)
        call_count = [0]

        def fail_on_second(cmd, name, rg, container_name, sitecontainer, slot):
            call_count[0] += 1
            if call_count[0] == 2:
                raise Exception("Simulated ARM failure")
            return MagicMock()

        from azure.cli.core.azclierror import AzureInternalError
        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal", side_effect=fail_on_second), \
             patch(f"{_CUSTOM_MOD}.update_app_settings"), \
             patch(f"{_CUSTOM_MOD}.update_site_configs"), \
             patch(f"{_CUSTOM_MOD}.delete_webapp_sitecontainer") as mock_delete, \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            with self.assertRaises(AzureInternalError):
                _convert_compose_to_sitecontainers(
                    cmd, "testapp", "testrg", None, site_config, linux_fx
                )
            # 'sidecar' < 'web' alphabetically, so sidecar is created first (succeeds),
            # then web fails. Rollback deletes the already-created 'sidecar'.
            mock_delete.assert_called_once()
            delete_args = mock_delete.call_args
            self.assertEqual(delete_args[0][1], "testapp")
            self.assertEqual(delete_args[0][3], "sidecar")

    def test_linuxfxversion_not_set_on_failure(self):
        """If container creation fails, linuxFxVersion should NOT be changed."""
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "nginx:alpine", "ports": ["80:80"]},
            }
        }
        cmd, raw_response = _make_mock_cmd()
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose)

        from azure.cli.core.azclierror import AzureInternalError
        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal",
                   side_effect=Exception("ARM failure")), \
             patch(f"{_CUSTOM_MOD}.update_app_settings"), \
             patch(f"{_CUSTOM_MOD}.update_site_configs") as mock_update_config, \
             patch(f"{_CUSTOM_MOD}.delete_webapp_sitecontainer"), \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            with self.assertRaises(AzureInternalError):
                _convert_compose_to_sitecontainers(
                    cmd, "testapp", "testrg", None, site_config, linux_fx
                )
            mock_update_config.assert_not_called()


# ---------------------------------------------------------------------------
# Orchestration: Auth type detection
# ---------------------------------------------------------------------------
class TestAuthTypeDetection(unittest.TestCase):

    def _run_and_capture_auth(self, compose_dict, site_config_kwargs=None, existing_settings=None):
        """Run conversion and capture the auth_type used for each container."""
        cmd, raw_response = _make_mock_cmd(existing_settings)
        site_config = _make_mock_site_config(**(site_config_kwargs or {}))
        linux_fx = _make_compose_b64(compose_dict)
        created = []

        def track_create(cmd, name, rg, container_name, sitecontainer, slot):
            created.append({
                "container_name": container_name,
                "auth_type": sitecontainer.auth_type,
                "user_name": sitecontainer.user_name,
                "user_managed_identity_client_id": sitecontainer.user_managed_identity_client_id,
            })
            return MagicMock()

        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal", side_effect=track_create), \
             patch(f"{_CUSTOM_MOD}.update_app_settings"), \
             patch(f"{_CUSTOM_MOD}.update_site_configs"), \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            _convert_compose_to_sitecontainers(
                cmd, "testapp", "testrg", None, site_config, linux_fx
            )
        return created

    def test_anonymous_when_no_credentials(self):
        compose = {"version": "3", "services": {"web": {"image": "nginx:alpine", "ports": ["80:80"]}}}
        created = self._run_and_capture_auth(compose)
        self.assertEqual(created[0]["auth_type"], "Anonymous")

    def test_user_credentials_from_app_settings(self):
        compose = {"version": "3", "services": {"web": {"image": "myacr.azurecr.io/app:v1", "ports": ["80:80"]}}}
        created = self._run_and_capture_auth(compose, existing_settings={
            "DOCKER_REGISTRY_SERVER_USERNAME": "myuser",
            "DOCKER_REGISTRY_SERVER_PASSWORD": "mypass",
        })
        self.assertEqual(created[0]["auth_type"], "UserCredentials")
        self.assertEqual(created[0]["user_name"], "myuser")

    def test_system_identity(self):
        compose = {"version": "3", "services": {"web": {"image": "myacr.azurecr.io/app:v1", "ports": ["80:80"]}}}
        created = self._run_and_capture_auth(compose, site_config_kwargs={
            "acr_use_managed_identity_creds": True,
        })
        self.assertEqual(created[0]["auth_type"], "SystemIdentity")

    def test_user_assigned_managed_identity(self):
        compose = {"version": "3", "services": {"web": {"image": "myacr.azurecr.io/app:v1", "ports": ["80:80"]}}}
        created = self._run_and_capture_auth(compose, site_config_kwargs={
            "acr_use_managed_identity_creds": True,
            "acr_user_managed_identity_id": "client-id-123",
        })
        self.assertEqual(created[0]["auth_type"], "UserAssigned")
        self.assertEqual(created[0]["user_managed_identity_client_id"], "client-id-123")

    def test_auth_shared_across_all_containers(self):
        compose = {
            "version": "3",
            "services": {
                "web": {"image": "myacr.azurecr.io/web:v1", "ports": ["80:80"]},
                "worker": {"image": "myacr.azurecr.io/worker:v1"},
            }
        }
        created = self._run_and_capture_auth(compose, existing_settings={
            "DOCKER_REGISTRY_SERVER_USERNAME": "user",
            "DOCKER_REGISTRY_SERVER_PASSWORD": "pass",
        })
        self.assertEqual(len(created), 2)
        for c in created:
            self.assertEqual(c["auth_type"], "UserCredentials")


# ---------------------------------------------------------------------------
# Orchestration: Invalid compose input
# ---------------------------------------------------------------------------
class TestInvalidComposeInput(unittest.TestCase):

    def _run(self, linux_fx):
        cmd, raw_response = _make_mock_cmd()
        site_config = _make_mock_site_config()
        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"):
            _convert_compose_to_sitecontainers(
                cmd, "testapp", "testrg", None, site_config, linux_fx
            )

    def test_invalid_base64_raises(self):
        from azure.cli.core.azclierror import ValidationError
        with self.assertRaises(ValidationError) as ctx:
            self._run("COMPOSE|!!!not-base64!!!")
        self.assertIn("base64", str(ctx.exception).lower())

    def test_missing_services_raises(self):
        from azure.cli.core.azclierror import ValidationError
        compose = {"version": "3"}  # no services key
        linux_fx = _make_compose_b64(compose)
        with self.assertRaises(ValidationError) as ctx:
            self._run(linux_fx)
        self.assertIn("services", str(ctx.exception).lower())

    def test_empty_services_raises(self):
        from azure.cli.core.azclierror import ValidationError
        compose = {"version": "3", "services": {}}
        linux_fx = _make_compose_b64(compose)
        with self.assertRaises(ValidationError) as ctx:
            self._run(linux_fx)
        self.assertIn("no services", str(ctx.exception).lower())

    def test_service_without_image_raises(self):
        from azure.cli.core.azclierror import ValidationError
        compose = {"version": "3", "services": {"web": {"ports": ["80:80"]}}}
        linux_fx = _make_compose_b64(compose)
        with self.assertRaises(ValidationError) as ctx:
            self._run(linux_fx)
        self.assertIn("image", str(ctx.exception).lower())


# ---------------------------------------------------------------------------
# Orchestration: linuxFxVersion set to SITECONTAINERS on success
# ---------------------------------------------------------------------------
class TestLinuxFxVersionSet(unittest.TestCase):

    def test_sitecontainers_fx_version_set_on_success(self):
        compose = {
            "version": "3",
            "services": {"web": {"image": "nginx:alpine", "ports": ["80:80"]}}
        }
        cmd, raw_response = _make_mock_cmd()
        site_config = _make_mock_site_config()
        linux_fx = _make_compose_b64(compose)

        with patch(f"{_CUSTOM_MOD}.send_raw_request", return_value=raw_response), \
             patch(_GET_SUB_ID, return_value="sub-id"), \
             patch(f"{_CUSTOM_MOD}._create_or_update_webapp_sitecontainer_internal", return_value=MagicMock()), \
             patch(f"{_CUSTOM_MOD}.update_app_settings"), \
             patch(f"{_CUSTOM_MOD}.update_site_configs") as mock_update_config, \
             patch(f"{_CUSTOM_MOD}.prompt_y_n", return_value=True):
            _convert_compose_to_sitecontainers(
                cmd, "testapp", "testrg", None, site_config, linux_fx
            )
            mock_update_config.assert_called_once_with(
                cmd, "testrg", "testapp", slot=None, linux_fx_version="SITECONTAINERS"
            )


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import sys

from knack.log import get_logger

logger = get_logger(__name__)


HOMEBREW_CELLAR_PATH = '/usr/local/Cellar/azure-cli/'


def is_homebrew():
    return any(p.startswith(HOMEBREW_CELLAR_PATH) for p in sys.path)


# A workaround for https://github.com/Azure/azure-cli/issues/4428
class HomebrewPipPatch:  # pylint: disable=too-few-public-methods

    CFG_FILE = os.path.expanduser(os.path.join('~', '.pydistutils.cfg'))

    def __init__(self):
        self.our_cfg_file = False

    def __enter__(self):
        if not is_homebrew():
            return
        if os.path.isfile(HomebrewPipPatch.CFG_FILE):
            logger.debug("Homebrew patch: The file %s already exists and we will not overwrite it. "
                         "If extension installation fails, temporarily rename this file and try again.",
                         HomebrewPipPatch.CFG_FILE)
            logger.warning("Unable to apply Homebrew patch for extension installation. "
                           "Attempting to continue anyway...")
            self.our_cfg_file = False
        else:
            logger.debug("Homebrew patch: Temporarily creating %s to support extension installation on Homebrew.",
                         HomebrewPipPatch.CFG_FILE)
            with open(HomebrewPipPatch.CFG_FILE, "w") as f:
                f.write("[install]\nprefix=")
            self.our_cfg_file = True

    def __exit__(self, exc_type, exc_value, tb):
        if not is_homebrew():
            return
        if self.our_cfg_file and os.path.isfile(HomebrewPipPatch.CFG_FILE):
            logger.debug("Homebrew patch: Deleting the temporarily created %s", HomebrewPipPatch.CFG_FILE)
            os.remove(HomebrewPipPatch.CFG_FILE)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Show(AAZCommand):
    """Get for Get an App Service plan.
    """

    _aaz_info = {
        "version": "2025-03-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.web/serverfarms/{}", "2025-03-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Name of the App Service plan.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.AppServicePlansGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AppServicePlansGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/serverfarms/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.extended_location = AAZObjectType(
                serialized_name="extendedLocation",
            )
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.identity = AAZIdentityObjectType()
            _schema_on_200.kind = AAZStrType()
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.sku = AAZObjectType()
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            extended_location = cls._schema_on_200.extended_location
            extended_location.name = AAZStrType()
            extended_location.type = AAZStrType(
                flags={"read_only": True},
            )

            identity = cls._schema_on_200.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType()
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_200.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType()

            _element = cls._schema_on_200.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.async_scaling_enabled = AAZBoolType(
                serialized_name="asyncScalingEnabled",
            )
            properties.elastic_scale_enabled = AAZBoolType(
                serialized_name="elasticScaleEnabled",
            )
            properties.free_offer_expiration_time = AAZStrType(
                serialized_name="freeOfferExpirationTime",
            )
            properties.geo_region = AAZStrType(
                serialized_name="geoRegion",
                flags={"read_only": True},
            )
            properties.hosting_environment_profile = AAZObjectType(
                serialized_name="hostingEnvironmentProfile",
            )
            properties.hyper_v = AAZBoolType(
                serialized_name="hyperV",
            )
            properties.install_scripts = AAZListType(
                serialized_name="installScripts",
            )
            properties.is_custom_mode = AAZBoolType(
                serialized_name="isCustomMode",
            )
            properties.is_spot = AAZBoolType(
                serialized_name="isSpot",
            )
            properties.is_xenon = AAZBoolType(
                serialized_name="isXenon",
            )
            properties.kube_environment_profile = AAZObjectType(
                serialized_name="kubeEnvironmentProfile",
            )
            properties.maximum_elastic_worker_count = AAZIntType(
                serialized_name="maximumElasticWorkerCount",
            )
            properties.maximum_number_of_workers = AAZIntType(
                serialized_name="maximumNumberOfWorkers",
                flags={"read_only": True},
            )
            properties.network = AAZObjectType()
            properties.number_of_sites = AAZIntType(
                serialized_name="numberOfSites",
                flags={"read_only": True},
            )
            properties.number_of_workers = AAZIntType(
                serialized_name="numberOfWorkers",
                flags={"read_only": True},
            )
            properties.per_site_scaling = AAZBoolType(
                serialized_name="perSiteScaling",
            )
            properties.plan_default_identity = AAZObjectType(
                serialized_name="planDefaultIdentity",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.rdp_enabled = AAZBoolType(
                serialized_name="rdpEnabled",
            )
            properties.registry_adapters = AAZListType(
                serialized_name="registryAdapters",
            )
            properties.reserved = AAZBoolType()
            properties.resource_group = AAZStrType(
                serialized_name="resourceGroup",
                flags={"read_only": True},
            )
            properties.spot_expiration_time = AAZStrType(
                serialized_name="spotExpirationTime",
            )
            properties.status = AAZStrType(
                flags={"read_only": True},
            )
            properties.storage_mounts = AAZListType(
                serialized_name="storageMounts",
            )
            properties.subscription = AAZStrType(
                flags={"read_only": True},
            )
            properties.target_worker_count = AAZIntType(
                serialized_name="targetWorkerCount",
            )
            properties.target_worker_size_id = AAZIntType(
                serialized_name="targetWorkerSizeId",
            )
            properties.worker_tier_name = AAZStrType(
                serialized_name="workerTierName",
            )
            properties.zone_redundant = AAZBoolType(
                serialized_name="zoneRedundant",
            )

            hosting_environment_profile = cls._schema_on_200.properties.hosting_environment_profile
            hosting_environment_profile.id = AAZStrType()
            hosting_environment_profile.name = AAZStrType(
                flags={"read_only": True},
            )
            hosting_environment_profile.type = AAZStrType(
                flags={"read_only": True},
            )

            install_scripts = cls._schema_on_200.properties.install_scripts
            install_scripts.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.install_scripts.Element
            _element.name = AAZStrType()
            _element.source = AAZObjectType()

            source = cls._schema_on_200.properties.install_scripts.Element.source
            source.source_uri = AAZStrType(
                serialized_name="sourceUri",
            )
            source.type = AAZStrType()

            kube_environment_profile = cls._schema_on_200.properties.kube_environment_profile
            kube_environment_profile.id = AAZStrType()
            kube_environment_profile.name = AAZStrType(
                flags={"read_only": True},
            )
            kube_environment_profile.type = AAZStrType(
                flags={"read_only": True},
            )

            network = cls._schema_on_200.properties.network
            network.virtual_network_subnet_id = AAZStrType(
                serialized_name="virtualNetworkSubnetId",
            )

            plan_default_identity = cls._schema_on_200.properties.plan_default_identity
            plan_default_identity.identity_type = AAZStrType(
                serialized_name="identityType",
            )
            plan_default_identity.user_assigned_identity_resource_id = AAZStrType(
                serialized_name="userAssignedIdentityResourceId",
            )

            registry_adapters = cls._schema_on_200.properties.registry_adapters
            registry_adapters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.registry_adapters.Element
            _element.key_vault_secret_reference = AAZObjectType(
                serialized_name="keyVaultSecretReference",
            )
            _ShowHelper._build_schema_key_vault_reference_with_status_read(_element.key_vault_secret_reference)
            _element.registry_key = AAZStrType(
                serialized_name="registryKey",
            )
            _element.type = AAZStrType()

            storage_mounts = cls._schema_on_200.properties.storage_mounts
            storage_mounts.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.storage_mounts.Element
            _element.credentials_key_vault_reference = AAZObjectType(
                serialized_name="credentialsKeyVaultReference",
            )
            _ShowHelper._build_schema_key_vault_reference_with_status_read(_element.credentials_key_vault_reference)
            _element.destination_path = AAZStrType(
                serialized_name="destinationPath",
            )
            _element.name = AAZStrType()
            _element.source = AAZStrType()
            _element.type = AAZStrType()

            sku = cls._schema_on_200.sku
            sku.capabilities = AAZListType()
            sku.capacity = AAZIntType()
            sku.family = AAZStrType()
            sku.locations = AAZListType()
            sku.name = AAZStrType()
            sku.size = AAZStrType()
            sku.sku_capacity = AAZObjectType(
                serialized_name="skuCapacity",
            )
            sku.tier = AAZStrType()

            capabilities = cls._schema_on_200.sku.capabilities
            capabilities.Element = AAZObjectType()

            _element = cls._schema_on_200.sku.capabilities.Element
            _element.name = AAZStrType()
            _element.reason = AAZStrType()
            _element.value = AAZStrType()

            locations = cls._schema_on_200.sku.locations
            locations.Element = AAZStrType()

            sku_capacity = cls._schema_on_200.sku.sku_capacity
            sku_capacity.default = AAZIntType()
            sku_capacity.elastic_maximum = AAZIntType(
                serialized_name="elasticMaximum",
            )
            sku_capacity.maximum = AAZIntType()
            sku_capacity.minimum = AAZIntType()
            sku_capacity.scale_type = AAZStrType(
                serialized_name="scaleType",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""

    _schema_key_vault_reference_with_status_read = None

    @classmethod
    def _build_schema_key_vault_reference_with_status_read(cls, _schema):
        if cls._schema_key_vault_reference_with_status_read is not None:
            _schema.reference_status = cls._schema_key_vault_reference_with_status_read.reference_status
            _schema.secret_uri = cls._schema_key_vault_reference_with_status_read.secret_uri
            return

        cls._schema_key_vault_reference_with_status_read = _schema_key_vault_reference_with_status_read = AAZObjectType()

        key_vault_reference_with_status_read = _schema_key_vault_reference_with_status_read
        key_vault_reference_with_status_read.reference_status = AAZStrType(
            serialized_name="referenceStatus",
        )
        key_vault_reference_with_status_read.secret_uri = AAZStrType(
            serialized_name="secretUri",
        )

        _schema.reference_status = cls._schema_key_vault_reference_with_status_read.reference_status
        _schema.secret_uri = cls._schema_key_vault_reference_with_status_read.secret_uri


__all__ = ["Show"]
